package com.io.yutian.thewardungeon.command.list;

import com.io.yutian.thewardungeon.dungeon.Dungeon;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.lang.Lang;
import com.io.yutian.thewardungeon.manager.DungeonManager;
import com.io.yutian.thewarskyblocklib.command.CommandContext;
import com.io.yutian.thewarskyblocklib.command.ICommand;
import com.io.yutian.thewarskyblocklib.command.argument.Argument;
import com.io.yutian.thewarskyblocklib.command.argument.ArgumentTypes;
import com.io.yutian.thewarskyblocklib.command.argument.Suggests;
import net.kyori.adventure.text.Component;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.entity.Player;

import java.util.Arrays;

public class CommandStart extends ICommand {

    public CommandStart() {
        super("start");
        addArgument(Argument.argument("id", ArgumentTypes.STRING).suggest(Suggests.DUNGEON_LIST));
        addArgument(Argument.argument("mode", ArgumentTypes.STRING).suggest(()-> Arrays.asList("start", "match", "room")));
        addArgument(Argument.argument("player", ArgumentTypes.STRING).suggest(Suggests.PLAYER_LIST).optional(null));
    }

    @Override
    public void executes(CommandContext commandContext) {
        CommandSender sender = commandContext.getSender();
        String id = commandContext.getArgumentsValue("id").getString();
        String mode = commandContext.getArgumentsValue("mode").getString();
        String playerName = commandContext.getArgumentsValue("player").getString();
        Dungeon dungeon = DungeonManager.getDungeon(id);
        if (dungeon == null) {
            sender.sendMessage(Component.text(Lang.prefix+"§c副本 "+id+" 不存在"));
            return;
        }
        if (playerName == null && sender instanceof ConsoleCommandSender) {
            sender.sendMessage(Lang.prefix+"§c请输入玩家ID");
            return;
        }
        Player player = null;
        if (sender instanceof Player) {
            player = (Player) sender;
        } else if (sender instanceof ConsoleCommandSender) {
            Player player1 = Bukkit.getPlayer(playerName);
            player = player1;
        }
        if (player == null || !player.isOnline()) {
            sender.sendMessage(Lang.prefix+Lang.get("command-player-noonline"));
            return;
        }
        if (DungeonManager.getDungeonInstance(player) != null) {
            if (player.equals(sender)) {
                player.sendMessage(Lang.prefix+"§f你已在副本中");
            } else {
                sender.sendMessage(Lang.prefix+"§f该玩家已在副本中");
            }
            return;
        }
        mode = mode.toLowerCase();
        if (mode.equalsIgnoreCase("start")) {
            if (!dungeon.getConfig().getConditionGroup().isValid(player) && !player.isOp()) {
                return;
            }
            if (dungeon.getConfig().getOption().getMin() != 1) {
                player.sendMessage(Lang.prefix+"§c该副本无法单独开始");
                return;
            }
            if (!dungeon.hasFreeInstance()) {
                player.sendMessage(Lang.prefix+"§c当前队列已满,请稍等");
                return;
            }
            DungeonInstance instance = dungeon.newInstance();
            instance.join(player, true);
        } else if (mode.equalsIgnoreCase("match")) {
            if (!dungeon.getConfig().getConditionGroup().isValid(player)) {
                return;
            }
            DungeonInstance dungeonInstance = dungeon.getQueue().findFreeDungeonInstance();
            if (dungeonInstance == null) {
                player.sendMessage(Lang.prefix+"§c未找到空闲房间");
                return;
            }
            dungeonInstance.join(player, false);
        } else if (mode.equalsIgnoreCase("room")) {
            if (!dungeon.getConfig().getConditionGroup().isValid(player)) {
                return;
            }
            if (!dungeon.hasFreeInstance()) {
                player.sendMessage(Lang.prefix+"§c当前队列已满,请稍等");
                return;
            }
            DungeonInstance instance = dungeon.newInstance();
            instance.join(player, false);
        } else {
            sender.sendMessage(Lang.prefix+"§c未知的模式");
        }
    }

}
