package com.io.yutian.thewardungeon.dungeon;

import com.io.yutian.thewardungeon.TheWarDungeon;
import com.io.yutian.thewardungeon.dungeon.config.DungeonConfig;
import com.io.yutian.thewardungeon.dungeon.queue.DungeonQueue;
import com.io.yutian.thewardungeon.dungeon.world.WorldInstance;
import com.io.yutian.thewardungeon.util.Log;
import com.io.yutian.thewarskyblocklib.util.FileUtil;
import org.bukkit.GameRule;
import org.bukkit.World;
import org.bukkit.WorldCreator;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

public class Dungeon {

    private final String id;
    private final DungeonConfig config;

    private File mapFolder;

    private final DungeonQueue queue;
    private List<WorldInstance> worldInstances;

    public Dungeon(String id, DungeonConfig config) {
        this.id = id;
        this.config = config;
        this.config.read();
        this.queue = new DungeonQueue(this, config.getOption().getMaxInstance());
        this.worldInstances = new LinkedList<>();
        this.mapFolder = new File(TheWarDungeon.inst().getDataFolder()+File.separator+"maps"+File.separator+config.getMap());
    }

    public String getId() {
        return id;
    }

    public DungeonConfig getConfig() {
        return config;
    }

    public File getMapFolder() {
        return mapFolder;
    }

    public boolean hasFreeInstance() {
        if (worldInstances.size() < config.getOption().getMaxInstance()) {
            return true;
        }
        for (WorldInstance worldInstance : worldInstances) {
            if (!worldInstance.isOccupied()) {
                return true;
            }
        }
        return false;
    }

    public WorldInstance getFreeInstance() {
        for (WorldInstance worldInstance : worldInstances) {
            if (!worldInstance.isOccupied()) {
                return worldInstance;
            }
        }
        if (worldInstances.size() >= config.getOption().getMaxInstance()) {
            return null;
        }
        if (!mapFolder.exists()) {
            Log.error("副本Map文件不存在"+ mapFolder);
            return null;
        }
        UUID uuid = UUID.randomUUID();
        String worldName = "dw_"+id + "_"+ uuid.toString().replace("_", "");
        File folder = new File(worldName);
        folder.mkdirs();
        FileUtil.copyDir(mapFolder, folder);
        File tFile = new File(folder+File.separator+"dungeonworld.token");
        if (!tFile.exists()) {
            try {
                tFile.createNewFile();
            } catch (Exception e) {
            }
        }
        World world;
        WorldCreator dungeonWorldCreator = new WorldCreator(worldName);
        try {
            world = dungeonWorldCreator.createWorld();
            initWorld(world);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
        WorldInstance worldInstance = new WorldInstance(folder, world, uuid);
        worldInstances.add(worldInstance);
        return worldInstance;
    }

    private void initWorld(World world) {
        world.setGameRule(GameRule.KEEP_INVENTORY, true);
        world.setGameRule(GameRule.MOB_GRIEFING, false);
        world.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        world.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        world.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        world.setKeepSpawnInMemory(false);
        world.setAutoSave(false);
    }

    @Nullable
    public DungeonInstance newInstance() {
        if (!hasFreeInstance()) {
            return null;
        }
        WorldInstance worldInstance = getFreeInstance();
        DungeonInstance dungeonInstance = new DungeonInstance(worldInstance.getUUID(), this, worldInstance);
        worldInstance.setOccupied(true);
        queue.add(dungeonInstance);
        return dungeonInstance;
    }

    public DungeonQueue getQueue() {
        return queue;
    }

    @Override
    public String toString() {
        return "Dungeon["+id+"]";
    }

}
