package com.io.yutian.thewardungeon.dungeon;

import com.io.yutian.thewardungeon.TheWarDungeon;
import com.io.yutian.thewardungeon.dungeon.objective.Objective;
import com.io.yutian.thewardungeon.dungeon.objective.ObjectiveGourp;
import com.io.yutian.thewardungeon.dungeon.objective.list.LocationObjective;
import com.io.yutian.thewardungeon.dungeon.objective.list.RegionObjective;
import com.io.yutian.thewardungeon.dungeon.objective.list.TimeObjective;
import com.io.yutian.thewardungeon.dungeon.player.DungeonPlayer;
import com.io.yutian.thewardungeon.dungeon.player.PlayerState;
import com.io.yutian.thewardungeon.dungeon.world.WorldInstance;
import com.io.yutian.thewardungeon.lang.Lang;
import com.io.yutian.thewardungeon.manager.DungeonManager;
import com.io.yutian.thewardungeon.script.ScriptGourp;
import com.io.yutian.thewardungeon.util.BossBarUtil;
import com.io.yutian.thewardungeon.util.LogUtil;
import com.io.yutian.thewarskyblocklib.math.Point;
import io.lumine.mythic.bukkit.BukkitAdapter;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.core.mobs.ActiveMob;
import io.lumine.mythic.core.mobs.MobRegistry;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.boss.BarColor;
import org.bukkit.boss.BarStyle;
import org.bukkit.boss.BossBar;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;
import org.bukkit.scheduler.BukkitTask;

import java.util.*;

public class DungeonInstance {

    protected final UUID uuid;
    private final Dungeon dungeon;
    private final WorldInstance worldInstance;

    private int id;

    private DungeonState state = DungeonState.WAITING;

    private BossBar bossBar;

    private DungeonLobby dungeonLobby;

    private Point spawnPoint;

    private Map<UUID, DungeonPlayer> players = new HashMap<>();
    private List<ObjectiveGourp> objectiveGourps = new ArrayList<>();

    private int time;

    protected int totalCountdown = -1;
    protected int countdown = -1;

    private List<ActiveMob> activeMobs = new ArrayList<>();

    private List<BukkitTask> tasks = new ArrayList<>();

    private boolean loaded = false;
    private boolean overdue = false;

    public DungeonInstance(UUID uuid, Dungeon dungeon, WorldInstance worldInstance) {
        this.uuid = uuid;
        this.dungeon = dungeon;
        this.worldInstance = worldInstance;
        this.bossBar = Bukkit.createBossBar("倒计时", BarColor.WHITE, BarStyle.SOLID);
    }

    public void join(Player player, boolean start) {
        if (!canJoin(player)) {
            return;
        }
        this.bossBar.addPlayer(player);
        addPlayer(player);
        if (start && players.size() == 1) {
            player.teleport(dungeon.getConfig().getLobbyPoint().toLocation(worldInstance.getWorld()));
            Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), this::start, 10L);
        } else {
            player.teleport(dungeon.getConfig().getLobbyPoint().toLocation(worldInstance.getWorld()));
            if (dungeonLobby == null) {
                this.dungeonLobby = new DungeonLobby(this);
                this.dungeonLobby.runTaskTimer(TheWarDungeon.inst(), 0, 20L);
            }
            dungeonLobby.showBossBar(player);
        }
        if (players.size() == dungeon.getConfig().getOption().getMax()) {
            if (dungeonLobby != null) {
                dungeonLobby.setCounter(10);
                for (DungeonPlayer dungeonPlayer : getPlayers()) {
                    dungeonPlayer.getPlayer().sendMessage(Lang.prefix+Lang.get("dungeon.lobby.max"));
                }
            }
        }
        for (DungeonPlayer dungeonPlayer : getPlayers()) {
            Player player1 = dungeonPlayer.getPlayer();
            player1.sendMessage(Lang.prefix+Lang.get("dungeon.lobby.join", player.getName(), players.size(), dungeon.getConfig().getOption().getMax()));
        }
    }

    public void leave(Player player) {
        this.bossBar.removePlayer(player);
        if (dungeonLobby != null && state.equals(DungeonState.WAITING)) {
            dungeonLobby.removeBossBar(player);
        }
        DungeonPlayer dungeonPlayer = getPlayer(player);
        if (dungeonPlayer != null) {
            player.setGameMode(dungeonPlayer.getGameMode());
        }
        if (players.size() - 1 == 0) {
            if (dungeonLobby != null && dungeonLobby.isCancelled()) {
                dungeonLobby.cancel();
            }
            stop();
        } else {
            players.remove(player.getUniqueId());
            player.teleport(dungeon.getConfig().getEndPoint());
        }
    }


    public void respawn(DungeonPlayer player) {
        if (player.getDeath() > dungeon.getConfig().getOption().getReviveAmount()) {
            return;
        }
        if (player.getState().equals(PlayerState.DEAD)) {
            player.setState(PlayerState.LIVING);
        }
        player.getPlayer().setGameMode(dungeon.getConfig().getOption().getGameMode());
        World world = worldInstance.getWorld();
        Location respawnLocation = dungeon.getConfig().getStartPoint().toLocation(world);
        if (spawnPoint != null) {
            respawnLocation = spawnPoint.toLocation(world);
        }
        player.getPlayer().teleport(respawnLocation);
    }

    public boolean canJoin(Player player) {
        return state.equals(DungeonState.WAITING) && players.size() < dungeon.getConfig().getOption().getMax();
    }

    private void addPlayer(Player player) {
        if (!state.equals(DungeonState.WAITING)) {
            return;
        }
        players.put(player.getUniqueId(), new DungeonPlayer(player));
    }

    public void setState(DungeonState state) {
        this.state = state;
    }

    public void addTask(BukkitTask task) {
        if (overdue) {
            return;
        }
        tasks.add(task);
    }

    public void addActiveMob(ActiveMob activeMob) {
        activeMobs.add(activeMob);
    }

    public List<ActiveMob> getActiveMobs() {
        return activeMobs;
    }

    public Dungeon getDungeon() {
        return dungeon;
    }

    public List<ObjectiveGourp> getObjectiveGourps() {
        return objectiveGourps;
    }

    public void checkObjectiveGroup() {
        List<ScriptGourp> scriptGourps = new ArrayList<>();
        Iterator<ObjectiveGourp> gourpIterator = objectiveGourps.iterator();
        while (gourpIterator.hasNext()) {
            ObjectiveGourp gourp = gourpIterator.next();
            boolean flag = true;
            for (Objective objective : gourp.getObjectives()) {
                if (!objective.isComplete()) {
                    flag = false;
                }
            }
            if (flag) {
                ScriptGourp scriptGourp = new ScriptGourp(gourp.getScripts());
                scriptGourps.add(scriptGourp);
                gourpIterator.remove();
            }
        }
        scriptGourps.forEach(scriptGourp -> scriptGourp.execute(this));
    }

    public void addObjectiveGourp(ObjectiveGourp gourp) {
        this.objectiveGourps.add(gourp);
        for (TimeObjective objective : gourp.getObjectives(TimeObjective.class)) {
            addTask(objective.createTask(this));
        }
    }

    public WorldInstance getWorldInstance() {
        return worldInstance;
    }

    public int getTime() {
        return time;
    }

    public DungeonState getState() {
        return state;
    }

    public List<DungeonPlayer> getPlayers() {
        List<DungeonPlayer> list = new ArrayList<>();
        players.values().forEach(list::add);
        return Collections.unmodifiableList(list);
    }

    public DungeonPlayer getPlayer(Player player) {
        return players.get(player.getUniqueId());
    }

    public void start() {
        if (overdue || worldInstance.isDestroyed()) {
            return;
        }
        boolean flag = true;
        while (flag) {
            if (!isWorldLoaded()) {
                flag = true;
                break;
            }
            state = DungeonState.RUNNING;
            countdown = dungeon.getConfig().getOption().getCountdown();
            for (DungeonPlayer dungeonPlayer : getPlayers()) {
                dungeonPlayer.getPlayer().setGameMode(dungeon.getConfig().getOption().getGameMode());
                dungeonPlayer.getPlayer().teleport(dungeon.getConfig().getStartPoint().toLocation(worldInstance.getWorld()));
            }
            Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), ()-> dungeon.getConfig().getStartScript().execute(this), 5L);
            tasks.add(new BukkitRunnable() {
                @Override
                public void run() {
                    if (overdue) {
                        cancel();
                        return;
                    }
                    time++;
                }
            }.runTaskTimer(TheWarDungeon.inst(), 0L, 1L));
            tasks.add(new BukkitRunnable() {
                @Override
                public void run() {
                    if (overdue) {
                        cancel();
                        return;
                    }
                    if (countdown > 0) {
                        countdown--;
                        if (countdown == 0) {
                            stop();
                            cancel();
                        }
                    }
                    updateBossBar();
                }
            }.runTaskTimer(TheWarDungeon.inst(), 20L, 20L));
            tasks.add(new BukkitRunnable() {
                @Override
                public void run() {
                    for (DungeonPlayer dungeonPlayer : getPlayers()) {
                        Player player = dungeonPlayer.getPlayer();
                        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(player.getWorld());
                        if (dungeonInstance == null || !dungeonInstance.getState().equals(DungeonState.RUNNING)) {
                            return;
                        }
                        for (ObjectiveGourp gourp : dungeonInstance.getObjectiveGourps()) {
                            List<LocationObjective> objectiveList1 = gourp.getObjectives(LocationObjective.class);
                            if (objectiveList1.size() > 0) {
                                for (LocationObjective locationObjective : objectiveList1) {
                                    if (locationObjective.isArrive(dungeonInstance, player)) {
                                        locationObjective.call();
                                    }
                                }
                            }
                            List<RegionObjective> objectiveList2 = gourp.getObjectives(RegionObjective.class);
                            if (objectiveList2.size() > 0) {
                                for (RegionObjective regionObjective : objectiveList2) {
                                    if (regionObjective.isInRegion(dungeonInstance, player)) {
                                        regionObjective.call();
                                    }
                                }
                            }
                        }
                        dungeonInstance.checkObjectiveGroup();
                    }
                }
            }.runTaskTimer(TheWarDungeon.inst(), 0L, 10L));
            flag = false;
        }
    }

    private void updateBossBar() {
        bossBar.setTitle("倒计时: " + countdown + "/" + dungeon.getConfig().getOption().getCountdown());
        float progress = (float) countdown / (float) dungeon.getConfig().getOption().getCountdown();
        bossBar.setProgress(progress);
        bossBar.setColor(BossBarUtil.getBarColor(progress));
    }

    private boolean isWorldLoaded() {
        try {
            BukkitAdapter.adapt(worldInstance.getWorld());
            return true;
        } catch (Exception e) {
            return false;
        }
    }

    public void stop() {
        if (overdue) {
            return;
        }
        overdue = true;
        state = DungeonState.END;
        LogUtil.logTimeMap.remove(this.getUUID());

        Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), ()-> {
            MobRegistry mobRegistry = MythicBukkit.inst().getMobManager().getMobRegistry();
            for (Entity entity : worldInstance.getWorld().getEntities()) {
                if (entity instanceof Player) {
                    continue;
                }
                entity.remove();
            }
            for (ActiveMob activeMob : activeMobs) {
                activeMob.setDespawned();
                MythicBukkit.inst().getMobManager().unregisterActiveMob(activeMob);
            }
        }, 15L);

        dungeon.getConfig().getEndScript().execute(this);
        tasks.forEach(bukkitTask -> {
            if (!bukkitTask.isCancelled()) {
                bukkitTask.cancel();
            }
        });
        Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), ()-> {
            for (DungeonPlayer dungeonPlayer : getPlayers()) {
                if (dungeonPlayer.getPlayer() != null && dungeonPlayer.getPlayer().isOnline()) {
                    dungeonPlayer.getPlayer().setGameMode(dungeonPlayer.getGameMode());
                    dungeonPlayer.getPlayer().teleport(dungeon.getConfig().getEndPoint());
                }
            }
            worldInstance.setOccupied(false);
        }, 20L);
    }

    public boolean isOverdue() {
        return overdue;
    }

    public UUID getUUID() {
        return uuid;
    }

    public int getCountdown() {
        return countdown;
    }

    public void setSpawnPoint(Point spawnPoint) {
        this.spawnPoint = spawnPoint;
    }

    public Point getSpawnPoint() {
        return spawnPoint;
    }

    public void setDungeonLobby(DungeonLobby dungeonLobby) {
        this.dungeonLobby = dungeonLobby;
    }

    public DungeonLobby getDungeonLobby() {
        return dungeonLobby;
    }

    public boolean isLoaded() {
        return loaded;
    }

    public void setLoaded(boolean loaded) {
        this.loaded = loaded;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DungeonInstance that = (DungeonInstance) o;
        return Objects.equals(uuid, that.uuid);
    }

    public void setId(int id) {
        this.id = id;
    }

    public int getId() {
        return id;
    }

    @Override
    public int hashCode() {
        return Objects.hash(uuid);
    }

    public int getTotalCountdown() {
        return totalCountdown;
    }

    public void setTotalCountdown(int totalCountdown) {
        this.totalCountdown = totalCountdown;
    }

    public void setCountdown(int countdown) {
        this.countdown = countdown;
    }

}
