package com.io.yutian.thewardungeon.dungeon;

import com.io.yutian.thewardungeon.dungeon.player.DungeonPlayer;
import com.io.yutian.thewardungeon.lang.Lang;
import net.kyori.adventure.bossbar.BossBar;
import net.kyori.adventure.text.Component;
import org.bukkit.Sound;
import org.bukkit.entity.Player;
import org.bukkit.scheduler.BukkitRunnable;

import java.util.List;

public class DungeonLobby extends BukkitRunnable {

    private DungeonInstance dungeonInstance;

    private int counter;
    private int maxTime;

    private BossBar bossBar;

    public DungeonLobby(DungeonInstance dungeonInstance) {
        this.dungeonInstance = dungeonInstance;
        this.maxTime = dungeonInstance.getDungeon().getConfig().getOption().getLobbyWaitTime();
        this.counter = maxTime;
        this.bossBar = BossBar.bossBar(Component.text(Lang.get("dungeon.lobby.bossbar", counter)), 1.0F, BossBar.Color.WHITE, BossBar.Overlay.PROGRESS);
    }

    public void showBossBar(Player player) {
        player.showBossBar(bossBar);
    }

    public void removeBossBar(Player player) {
        player.hideBossBar(bossBar);
    }

    @Override
    public void run() {
        List<DungeonPlayer> players = this.dungeonInstance.getPlayers();
        if (this.dungeonInstance.getState() != DungeonState.WAITING) {
            this.dungeonInstance.setDungeonLobby(null);
            this.cancel();
            return;
        }
        bossBar.progress((float) ((double) counter / (double) maxTime));
        bossBar.name(Component.text(Lang.get("dungeon.lobby.bossbar", counter)));
        if (this.counter == 15) {
            for (DungeonPlayer dungeonPlayer : players) {
                Player player = dungeonPlayer.getPlayer();
                player.sendMessage(Lang.prefix+Lang.get("dungeon.lobby.time-15"));
                player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1.0f, 1.0f);
            }
        }

        if (this.counter <= 3 && this.counter > 0) {
            for (DungeonPlayer dungeonPlayer : players) {
                Player player = dungeonPlayer.getPlayer();
                player.sendMessage(Lang.prefix+Lang.get("dungeon.lobby.time-lessthan-3", this.counter));
                player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1.0f, 1.0f);
            }
        }
        if (this.counter == 0) {
            if (dungeonInstance.getPlayers().size() < dungeonInstance.getDungeon().getConfig().getOption().getMin()) {
                this.counter = maxTime;
                for (DungeonPlayer dungeonPlayer : players) {
                    Player player = dungeonPlayer.getPlayer();
                    player.sendMessage(Lang.prefix+Lang.get("dungeon.lobby.min-reset"));
                    player.playSound(player.getLocation(), Sound.UI_BUTTON_CLICK, 1.0f, 1.0f);
                }
                return;
            }
            start();
            return;
        }
        this.counter--;

    }

    public void setCounter(int counter) {
        this.counter = counter;
    }

    public void start() {
        dungeonInstance.setDungeonLobby(null);
        for (DungeonPlayer dungeonPlayer : this.dungeonInstance.getPlayers()) {
            Player player = dungeonPlayer.getPlayer();
            removeBossBar(player);
        }
        this.dungeonInstance.start();
        this.cancel();
    }

}
