package com.io.yutian.thewardungeon.dungeon.config;

import com.io.yutian.thewardungeon.dungeon.condition.ConditionGroup;
import com.io.yutian.thewardungeon.dungeon.condition.ICondition;
import com.io.yutian.thewardungeon.dungeon.objective.Objective;
import com.io.yutian.thewardungeon.dungeon.objective.ObjectiveGourp;
import com.io.yutian.thewardungeon.script.Script;
import com.io.yutian.thewardungeon.script.ScriptGourp;
import com.io.yutian.thewardungeon.util.ConsoleColor;
import com.io.yutian.thewardungeon.util.Log;
import com.io.yutian.thewarskyblocklib.math.DirectionPoint;
import com.io.yutian.thewarskyblocklib.math.Point;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;

import java.io.File;
import java.util.*;

public class DungeonConfig {

    protected File folder;

    private String map;
    private DirectionPoint lobbyPoint;
    private DirectionPoint startPoint;
    private Location endPoint;

    private DungeonOption option = new DungeonOption();

    private ScriptGourp startScript;
    private ScriptGourp endScript;
    private Map<Integer, ScriptGourp> timingScripts;

    private ObjectiveGourp defaultObjecive;
    private ConditionGroup conditionGroup;

    private Map<String, ScriptGourp> scriptGourpMap = new HashMap<>();
    private Map<String, ObjectiveGourp> objectiveGourpMap = new HashMap<>();

    public DungeonConfig(File folder) {
        this.folder = folder;
    }

    public DungeonConfig(String map, File folder) {
        this.map = map;
        this.folder = folder;
    }

    public void read() {
        File configFile = new File(folder, "config.yml");
        File scriptFile = new File(folder, "script.yml");
        File objectiveFile = new File(folder, "objective.yml");
        File conditionFile = new File(folder, "condition.yml");
        if (!configFile.exists()) {
            initConfigFile();
        }
        if (!scriptFile.exists()) {
            initScriptFile();
        }
        if (!objectiveFile.exists()) {
            initObjectiveFile();
        }
        if (!conditionFile.exists()) {
            initConditionFile();
        }
        FileConfiguration configFileConfiguration = YamlConfiguration.loadConfiguration(configFile);
        FileConfiguration scriptFileConfiguration = YamlConfiguration.loadConfiguration(scriptFile);
        FileConfiguration objectiveFileConfiguration = YamlConfiguration.loadConfiguration(objectiveFile);
        FileConfiguration conditionFileConfiguration = YamlConfiguration.loadConfiguration(conditionFile);

        //            config.yml            //
        map = configFileConfiguration.getString("map");
        lobbyPoint = configFileConfiguration.contains("location.lobby") && configFileConfiguration.isConfigurationSection("location.lobby") ? DirectionPoint.deserialize(configFileConfiguration.getConfigurationSection("location.lobby")) : null;
        startPoint = configFileConfiguration.contains("location.start") && configFileConfiguration.isConfigurationSection("location.start") ? DirectionPoint.deserialize(configFileConfiguration.getConfigurationSection("location.start")) : null;
        if (configFileConfiguration.contains("location.end") && configFileConfiguration.isConfigurationSection("location.end")) {
            ConfigurationSection configurationSection = configFileConfiguration.getConfigurationSection("location.end");
            World world = Bukkit.getWorld(configurationSection.getString("world"));
            if (world != null) {
                double x = configurationSection.getDouble("x");
                double y = configurationSection.getDouble("y");
                double z = configurationSection.getDouble("z");
                float yaw = (float) configurationSection.getDouble("yaw", 0.0);
                float pitch = (float) configurationSection.getDouble("pitch", 0.0);
                endPoint = new Location(world, x, y, z, yaw, pitch);
            }
        }

        ConfigurationSection section0 = configFileConfiguration.getConfigurationSection("options");
        option.load(section0);
        //            config.yml            //

        //            script.yml            //
        for (String key : scriptFileConfiguration.getKeys(false)) {
            if (!scriptFileConfiguration.isList(key)) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: 错误的数据结构" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Location]: " + ConsoleColor.WHITE +"./dungeons/"+folder.getName()+"/script.yml");
                Log.error(ConsoleColor.RED + "[Key]: " + ConsoleColor.WHITE + key);
                continue;
            }
            List<String> lines = scriptFileConfiguration.getStringList(key);
            List<Script> scripts = new ArrayList<>();
            for (int i = 0; i < lines.size(); i++) {
                String line = lines.get(i);
                Script script = Script.getScript(line);
                scripts.add(script);
            }
            ScriptGourp scriptGourp = new ScriptGourp(scripts);
            if (key.equalsIgnoreCase("start")) {
                startScript = scriptGourp;
            } else if (key.equalsIgnoreCase("end")) {
                endScript = scriptGourp;
            } else {
                this.scriptGourpMap.put(key, scriptGourp);
            }
        }
        //            script.yml            //

        //            objective.yml            //
        for (String key : objectiveFileConfiguration.getKeys(false)) {
            if (!objectiveFileConfiguration.isConfigurationSection(key)) {
                Log.error(ConsoleColor.RED + "加载目标时发生错误: 错误的数据结构" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Location]: " + ConsoleColor.WHITE +"./dungeons/"+folder.getName()+"/objective.yml");
                Log.error(ConsoleColor.RED + "[Key]: " + ConsoleColor.WHITE + key);
                continue;
            }
            ConfigurationSection cs = objectiveFileConfiguration.getConfigurationSection(key);
            List<String> objectivesLines = cs.getStringList("objectives");
            List<String> scriptsLines = cs.getStringList("scripts");
            List<Objective> objectives = new ArrayList<>();

            List<Script> scripts = new ArrayList<>();
            for (int i = 0; i < objectivesLines.size(); i++) {
                String line = objectivesLines.get(i);
                Objective objective = Objective.getObjective(line);
                objectives.add(objective);
            }
            for (int i = 0; i < scriptsLines.size(); i++) {
                String line = scriptsLines.get(i);
                Script script = Script.getScript(line);
                scripts.add(script);
            }
            this.objectiveGourpMap.put(key, new ObjectiveGourp(objectives, scripts));
        }
        //            objective.yml            //

        //            condition.yml            //
        List<String> list = conditionFileConfiguration.getStringList("start");
        List<ICondition> conditions = new ArrayList<>();
        for (String s : list) {
            ICondition condition = ICondition.getCondition(s);
            if (condition != null) {
                conditions.add(condition);
            }
        }
        conditionGroup = new ConditionGroup(conditions);
        //            condition.yml            //

    }

    public void initConfig() {
        initConfigFile();
        initScriptFile();
        initObjectiveFile();
        initConditionFile();
    }

    protected void initConfigFile() {
        File file = new File(folder, "config.yml");
        try {
            file.createNewFile();
        } catch (Exception e) {
        }
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        fileConfiguration.set("map", map);
        ConfigurationSection section0 = fileConfiguration.createSection("location");
        section0.set("lobby.x", 0.0);
        section0.set("lobby.y", 0.0);
        section0.set("lobby.z", 0.0);
        section0.set("lobby.yaw", 0.0);
        section0.set("lobby.pitch", 0.0);

        section0.set("start.x", 0.0);
        section0.set("start.y", 0.0);
        section0.set("start.z", 0.0);
        section0.set("start.yaw", 0.0);
        section0.set("start.pitch", 0.0);

        section0.set("end.world", "world");
        section0.set("end.x", 0.0);
        section0.set("end.y", 0.0);
        section0.set("end.z", 0.0);
        section0.set("end.yaw", 0.0);
        section0.set("end.pitch", 0.0);

        ConfigurationSection section = fileConfiguration.createSection("options");
        section.set("min", 1);
        section.set("max", 8);
        section.set("lobbyWaitTime", 60);
        section.set("gamemode", GameMode.ADVENTURE.toString());
        section.set("revive", true);
        section.set("reviveAmount", -1);
        section.set("reviveTime", 10);
        section.set("breakBlock", false);
        section.set("placeBlock", false);
        section.set("pvp", false);
        try {
            fileConfiguration.save(file);
        } catch (Exception e) {
        }
    }

    protected void initScriptFile() {
        File file = new File(folder, "script.yml");
        try {
            file.createNewFile();
        } catch (Exception e) {
        }
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        fileConfiguration.set("start", Arrays.asList("setObjective{id=test}"));
        fileConfiguration.set("start", Arrays.asList("message{message=游戏开始}"));
        fileConfiguration.set("end", Arrays.asList("message{message=游戏结束}"));
        try {
            fileConfiguration.save(file);
        } catch (Exception e) {
        }
    }

    protected void initObjectiveFile() {
        File file = new File(folder, "objective.yml");
        try {
            file.createNewFile();
        } catch (Exception e) {
        }
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        fileConfiguration.set("test.objectives", Arrays.asList("killmob{type=mm:test;amount=1}"));
        fileConfiguration.set("test.scripts", Arrays.asList("message{message=游戏结束}", "stop"));
        try {
            fileConfiguration.save(file);
        } catch (Exception e) {
        }
    }

    protected void initConditionFile() {
        File file = new File(folder, "condition.yml");
        try {
            file.createNewFile();
        } catch (Exception e) {
        }
        FileConfiguration fileConfiguration = YamlConfiguration.loadConfiguration(file);
        fileConfiguration.set("start", Arrays.asList("condition{text=%player_level%>=30;message=你的等级不足30}"));
        try {
            fileConfiguration.save(file);
        } catch (Exception e) {
        }
    }

    public File getFolder() {
        return folder;
    }

    public String getMap() {
        return map;
    }

    public Point getLobbyPoint() {
        return lobbyPoint;
    }

    public Point getStartPoint() {
        return startPoint;
    }

    public Location getEndPoint() {
        return endPoint;
    }

    public ScriptGourp getStartScript() {
        return startScript;
    }

    public ScriptGourp getEndScript() {
        return endScript;
    }

    public Map<Integer, ScriptGourp> getTimingScripts() {
        return timingScripts;
    }

    public ObjectiveGourp getDefaultObjecive() {
        return defaultObjecive;
    }

    public ConditionGroup getConditionGroup() {
        return conditionGroup;
    }

    public ScriptGourp getScriptGourp(String id) {
        ScriptGourp scriptGourp = scriptGourpMap.getOrDefault(id, null);
        if (scriptGourp == null) {
            return null;
        }
        return scriptGourp.clone();
    }

    public ObjectiveGourp getObjectiveGourp(String id) {
        ObjectiveGourp gourp = objectiveGourpMap.getOrDefault(id, null);
        if (gourp == null) {
            return null;
        }
        return gourp.clone();
    }

    public DungeonOption getOption() {
        return option;
    }

}
