package com.io.yutian.thewardungeon.dungeon.objective;

import com.io.yutian.thewardungeon.data.LineMetadata;
import com.io.yutian.thewardungeon.manager.RegistryManager;
import com.io.yutian.thewardungeon.util.ConsoleColor;
import com.io.yutian.thewardungeon.util.Log;
import javassist.bytecode.Opcode;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

public abstract class Objective implements Cloneable {

    public LineMetadata metadata;

    private int progress;
    private int maxProgress = -1;

    public Objective(LineMetadata metadata) {
        this.metadata = metadata;
    }

    public void setProgress(int progress) {
        this.progress = progress;
    }

    public void setMaxProgress(int maxProgress) {
        this.maxProgress = maxProgress;
    }

    public void call() {
        if (maxProgress == -1) {
            return;
        }
        if (progress >= maxProgress) {
            return;
        }
        progress++;
    }

    public float getProgress() {
        return maxProgress == -1 ? 0.0F : progress / maxProgress;
    }

    public boolean isComplete() {
        return progress == maxProgress;
    }

    @Override
    public Objective clone() {
        try {
            return (Objective) super.clone();
        } catch (Exception e) {
            return this;
        }
    }

    public static Objective getObjective(String arg) {
        String key;
        String s = arg.toLowerCase();
        if (s.contains("{")) {
            key = s.substring(0, s.indexOf("{"));
        } else if (s.contains("[")) {
            key = s.substring(0, s.indexOf("["));
        } else {
            key = s;
        }
        Map<String, String> valueMap = new HashMap<>();
        String s2 = arg;
        if (s2.contains("{") && s2.contains("}")) {
            int count = 0;
            String s3 = s2.substring(s2.indexOf(Opcode.LSHR) + 1, s2.lastIndexOf(Opcode.LUSHR));
            char[] charArray = s3.toCharArray();
            int length = charArray.length;
            for (int i = 0; i < length; i++) {
                char c = charArray[i];
                count = c == '{' ? count + 1 : count;
                if (c == '}') {
                    count--;
                }
            }
            if (count != 0) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: 不平衡支撑" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + s3);
            } else if (s3.length() != 0) {
                int start = 0;
                int pos = 0;
                int depth = 0;
                String s4 = s3 + "}";
                char[] charArray2 = s4.toCharArray();
                int length2 = charArray2.length;
                for (int i2 = 0; i2 < length2; i2++) {
                    char c2 = charArray2[i2];
                    depth = (c2 == '}' || c2 == ']') ? ((c2 == '{' || c2 == '[') ? depth + 1 : depth) - 1 : depth;
                    if ((c2 == ';' && depth == 0) || (c2 == '}' && depth < 0)) {
                        try {
                            String element = s4.substring(start, pos);
                            if (pos - start > 0 && element.length() > 0) {
                                valueMap.put(element.substring(0, element.indexOf(61)).trim().toLowerCase(), element.substring(element.indexOf(61) + 1).trim());
                            }
                            start = pos + 1;
                        } catch (Exception e) {
                            start = pos + 1;
                        } catch (Throwable th) {
                            int i3 = pos + 1;
                            throw th;
                        }
                    }
                    pos++;
                }
            }
        } else if (!s2.contains("[") || !s2.contains("]")) {
            key = s2;
        } else {
            try {
                s2 = s2.split("\\[")[1].split("\\]")[0];
            } catch (ArrayIndexOutOfBoundsException e2) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: 无效语法" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + s2);
            }
        }
        key = key.toLowerCase();
        Class<? extends Objective> clazz = RegistryManager.getObjectiveRegistry().get(key);
        if (clazz == null) {
            return null;
        }
        try {
            Constructor constructor = clazz.getConstructor(LineMetadata.class);
            return (Objective) constructor.newInstance(new LineMetadata(arg, valueMap));
        } catch (Exception e) {
            return null;
        }
    }

}
