package com.io.yutian.thewardungeon.dungeon.objective.list;

import com.io.yutian.thewardungeon.data.LineMetadata;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.dungeon.objective.Objective;
import com.io.yutian.thewardungeon.util.LogUtil;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.core.mobs.ActiveMob;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.LivingEntity;

import java.util.Optional;

public class KillMobObjective extends Objective {

    private String type;
    private int amount;

    public KillMobObjective(LineMetadata metadata) {
        super(metadata);
        this.type = metadata.getString("type", "minecraft:zombie");
        this.amount = metadata.getInt("amount", 1);
        this.setMaxProgress(amount);
    }

    public boolean isValidEntity(DungeonInstance dungeonInstance, LivingEntity entity) {
        if (!type.contains(":") || type.startsWith(":") || type.endsWith(":")) {
            LogUtil.logObjective(dungeonInstance, metadata, KillMobObjective.class, "实体类型格式错误");
            return false;
        }
        String prefix = type.split(":")[0].toLowerCase();
        String type1 = type.split(":")[1];
        if (prefix.equalsIgnoreCase("minecraft")) {
            try {
                EntityType entityType = EntityType.valueOf(type1.toUpperCase());
                return entityType.equals(entity.getType());
            } catch (Exception e) {
                LogUtil.logObjective(dungeonInstance, metadata, KillMobObjective.class, "未知的实体类型");
                return false;
            }
        } else if (prefix.equalsIgnoreCase("mm") || prefix.equalsIgnoreCase("mythicmob")) {
            Optional<ActiveMob> optional = MythicBukkit.inst().getMobManager().getActiveMob(entity.getUniqueId());
            if (!optional.isPresent()) {
                return false;
            }
            ActiveMob activeMob = optional.get();
            return activeMob.getMobType().equalsIgnoreCase(type1);
        } else {
            LogUtil.logObjective(dungeonInstance, metadata, KillMobObjective.class, "未知的实体类型");
            return false;
        }
    }

    public String getType() {
        return type;
    }

    public int getAmount() {
        return amount;
    }

}
