package com.io.yutian.thewardungeon.dungeon.queue;

import com.io.yutian.thewardungeon.dungeon.Dungeon;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.dungeon.DungeonState;

import java.util.*;

public class DungeonQueue {

    private final Dungeon dungeon;
    private final int limit;

    private List<DungeonInstanceEntry> dungeonInstanceEntries = new ArrayList<>();

    public DungeonQueue(Dungeon dungeon) {
        this(dungeon, 10);
    }

    public DungeonQueue(Dungeon dungeon, int limit) {
        this.dungeon = dungeon;
        this.limit = limit;
    }

    public DungeonInstance findFreeDungeonInstance() {
        checkOverdue();
        List<DungeonInstance> list = new ArrayList<>();
        dungeonInstanceEntries.forEach(f->list.add(f.dungeonInstance));
        for (DungeonInstance dungeonInstance : list) {
            if (dungeonInstance.getState().equals(DungeonState.WAITING)) {
                if (dungeonInstance.getPlayers().size() < dungeonInstance.getDungeon().getConfig().getOption().getMax()) {
                    return dungeonInstance;
                }
            }
        }
        return null;
    }

    public List<DungeonInstance> getAllFreeDungeonInstance() {
        checkOverdue();
        List<DungeonInstance> list = new ArrayList<>();
        dungeonInstanceEntries.forEach(f->list.add(f.dungeonInstance));
        List<DungeonInstance> list1 = new ArrayList<>();
        for (DungeonInstance dungeonInstance : list) {
            if (dungeonInstance.getState().equals(DungeonState.WAITING)) {
                if (dungeonInstance.getPlayers().size() < dungeonInstance.getDungeon().getConfig().getOption().getMax()) {
                    list1.add(dungeonInstance);
                }
            }
        }
        return Collections.unmodifiableList(list1);
    }

    public List<DungeonInstance> getAllDungeonInstance() {
        checkOverdue();
        List<DungeonInstance> list = new ArrayList<>();
        dungeonInstanceEntries.forEach(f->list.add(f.dungeonInstance));
        return Collections.unmodifiableList(list);
    }

    public void add(DungeonInstance dungeonInstance) {
        checkOverdue();
        if (dungeonInstanceEntries.size() >= limit) {
            return;
        }
        int id = getNextId();
        if (id == -1) {
            return;
        }
        dungeonInstance.setId(id);
        dungeonInstanceEntries.add(new DungeonInstanceEntry(id, dungeonInstance));
    }

    public void clear() {
        dungeonInstanceEntries.clear();
    }

    public int size() {
        checkOverdue();
        return dungeonInstanceEntries.size();
    }

    public int getNextId() {
        checkOverdue();
        List<Integer> ids = new ArrayList<>();
        for (int i = 0; i < limit; i++) {
            ids.add(i);
        }
        for (DungeonInstanceEntry entry : dungeonInstanceEntries) {
            ids.remove(entry.id);
        }
        return ids.size() > 0 ? ids.get(0) : -1;
    }

    public void checkOverdue() {
        Iterator<DungeonInstanceEntry> iterator = dungeonInstanceEntries.iterator();
        while (iterator.hasNext()) {
            DungeonInstanceEntry instanceEntry = iterator.next();
            if (instanceEntry.dungeonInstance.isOverdue()) {
                iterator.remove();
            }
        }
    }

    private class DungeonInstanceEntry {

        private int id;
        private DungeonInstance dungeonInstance;

        public DungeonInstanceEntry(int id, DungeonInstance dungeonInstance) {
            this.id = id;
            this.dungeonInstance = dungeonInstance;
        }

    }

}
