package com.io.yutian.thewardungeon.listener;

import com.io.yutian.thewardungeon.TheWarDungeon;
import com.io.yutian.thewardungeon.config.CommonConfig;
import com.io.yutian.thewardungeon.dungeon.Dungeon;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.dungeon.player.DungeonPlayer;
import com.io.yutian.thewardungeon.dungeon.player.PlayerState;
import com.io.yutian.thewardungeon.lang.Lang;
import com.io.yutian.thewardungeon.manager.DungeonManager;
import net.minecraft.nbt.NBTCompressedStreamTools;
import net.minecraft.nbt.NBTTagCompound;
import org.bukkit.*;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.PlayerDeathEvent;
import org.bukkit.event.player.*;

import java.io.File;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;

public class DungeonListener implements Listener {

    private static Set<UUID> loginPlayers = new HashSet<>();

    @EventHandler(ignoreCancelled = true)
    public void onAsyncPlayerPreLogin(AsyncPlayerPreLoginEvent event) {
        UUID uuid = event.getUniqueId();
        File playerDataFile = new File("world"+File.separator+"playerdata"+File.separator+uuid+".dat");
        if (!playerDataFile.exists()) {
            return;
        }
        try {
            NBTTagCompound nbtTagCompound = NBTCompressedStreamTools.a(playerDataFile);
            String dimension = nbtTagCompound.l("Dimension");
            if (dimension != null && dimension.startsWith("minecraft:dw")) {
                loginPlayers.add(uuid);
            }
        } catch (Exception e) {
        }
    }

    @EventHandler(ignoreCancelled = true)
    public void onPlayerJoin(PlayerJoinEvent event) {
        Player player = event.getPlayer();
        if (loginPlayers.contains(player.getUniqueId())) {
            Location location = CommonConfig.safeLocation.get();
            if (location != null) {
                player.teleport(location);
            }
            loginPlayers.remove(player.getUniqueId());
        }
    }

    @EventHandler(ignoreCancelled = true)
    public void onPlayerCommandPreprocess(PlayerCommandPreprocessEvent event) {
        Player player = event.getPlayer();
        String command = event.getMessage();
        if (!isInDungeon(player) || player.isOp()) {
            return;
        }
        if (!command.startsWith("/")) {
            return;
        }
        command = command.substring(1);
        String mainCommand = command.toLowerCase();
        if (command.contains(" ")) {
            mainCommand = command.split(" ")[0];
        }
        if (!CommonConfig.allowCommands.get().contains(mainCommand)) {
            event.setCancelled(true);
            player.sendMessage(Lang.prefix+"§c在副本内无法使用该命令");
        }
    }

    @EventHandler(ignoreCancelled = true, priority = EventPriority.HIGHEST)
    public void onPlayerDeath(PlayerDeathEvent event) {
        Player player = event.getPlayer();
        if (!isInDungeon(player)) {
            return;
        }
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(player);
        if (dungeonInstance == null) {
            return;
        }
        DungeonPlayer dungeonPlayer = dungeonInstance.getPlayer(player);
        event.setCancelled(true);
        if (dungeonInstance.getDungeon().getConfig().getOption().getReviveAmount() == -1) {
            dungeonInstance.respawn(dungeonPlayer);
            return;
        }
        dungeonPlayer.addDeath(1);
        dungeonPlayer.setState(PlayerState.DEAD);
        if (dungeonPlayer.getDeath() <= dungeonInstance.getDungeon().getConfig().getOption().getReviveAmount()) {
            dungeonPlayer.sendRespawnMessage(dungeonInstance);
            Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), ()-> {
                if (!dungeonPlayer.getState().equals(PlayerState.DEAD)) {
                    return;
                }
                dungeonInstance.leave(player);
            }, dungeonInstance.getDungeon().getConfig().getOption().getReviveTime() * 20L);
        } else {
            dungeonPlayer.sendDeathMessage();
            Bukkit.getScheduler().runTaskLater(TheWarDungeon.inst(), ()-> {
                DungeonPlayer dungeonPlayer1 = dungeonInstance.getPlayer(player);
                if (dungeonPlayer1 == null && !dungeonPlayer1.equals(dungeonPlayer)) {
                    return;
                }
                dungeonInstance.leave(player);
            }, 5 * 20L);
        }
        player.setGameMode(GameMode.SPECTATOR);
        event.setCancelled(true);
    }

    @EventHandler(ignoreCancelled = true, priority = EventPriority.HIGHEST)
    public void onPlayerMove(PlayerMoveEvent event) {
        Player player = event.getPlayer();
        if (!isInDungeon(player)) {
            return;
        }
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(player);
        if (dungeonInstance == null) {
            return;
        }
        DungeonPlayer dungeonPlayer = dungeonInstance.getPlayer(player);
        if (dungeonPlayer.getState().equals(PlayerState.DEAD)) {
            event.setCancelled(true);
        }
    }

    @EventHandler(priority = EventPriority.LOWEST)
    public void onPlayerRespawn(PlayerRespawnEvent event) {
        Player player = event.getPlayer();
        if (!isInDungeon(player)) {
            return;
        }
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(player);
        World world = dungeonInstance.getWorldInstance().getWorld();
        Location respawnLocation = dungeonInstance.getDungeon().getConfig().getStartPoint().toLocation(world);
        if (dungeonInstance.getSpawnPoint() != null) {
            respawnLocation = dungeonInstance.getSpawnPoint().toLocation(world);
        }
        DungeonPlayer dungeonPlayer = dungeonInstance.getPlayer(player);
        if (dungeonPlayer.getState().equals(PlayerState.DEAD)) {
            dungeonPlayer.setState(PlayerState.LIVING);
        }
        event.setRespawnLocation(respawnLocation);
    }

    @EventHandler(ignoreCancelled = true)
    public void onBlockBreak(BlockBreakEvent event) {
        if (isDungeonWorld(event.getPlayer().getWorld())) {
            Dungeon dungeon = DungeonManager.getDungeon(event.getPlayer().getWorld());
            if (!dungeon.getConfig().getOption().isBreakBlock()) {
                event.setCancelled(true);
            }
        }
    }

    @EventHandler(ignoreCancelled = true)
    public void onBlockPlace(BlockPlaceEvent event) {
        if (isDungeonWorld(event.getPlayer().getWorld())) {
            Dungeon dungeon = DungeonManager.getDungeon(event.getPlayer().getWorld());
            if (!dungeon.getConfig().getOption().isPlaceBlock()) {
                event.setCancelled(true);
            }
        }
    }

    @EventHandler(ignoreCancelled = true)
    public void onEntityDamageByEntityEvent(EntityDamageByEntityEvent event) {
        if (event.getDamager() instanceof Player && event.getEntity() instanceof Player) {
            World world = event.getDamager().getWorld();
            if (isDungeonWorld(world)) {
                Dungeon dungeon = DungeonManager.getDungeon(world);
                if (!dungeon.getConfig().getOption().isPVP()) {
                    event.setCancelled(true);
                }
            }
        }
    }

    @EventHandler
    public void onPlayerQuit(PlayerQuitEvent event) {
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(event.getPlayer());
        if (dungeonInstance != null) {
            dungeonInstance.leave(event.getPlayer());
        }
    }

    private boolean isDungeonWorld(World world) {
        return world.getName().startsWith("dw_");
    }

    private boolean isInDungeon(Player player) {
        return DungeonManager.getDungeonInstance(player) != null;
    }

}
