package com.io.yutian.thewardungeon.listener;

import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.dungeon.DungeonState;
import com.io.yutian.thewardungeon.dungeon.objective.ObjectiveGourp;
import com.io.yutian.thewardungeon.dungeon.objective.list.KillMobObjective;
import com.io.yutian.thewardungeon.manager.DungeonManager;
import org.bukkit.World;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.entity.EntityDeathEvent;

import java.util.List;

public class ObjectiveListener implements Listener {

    @EventHandler
    public void onEntityDeath(EntityDeathEvent event) {
        LivingEntity entity = event.getEntity();
        if (!isDungeonWorld(entity.getWorld())) {
            return;
        }
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(entity.getWorld());
        if (dungeonInstance == null || !dungeonInstance.getState().equals(DungeonState.RUNNING)) {
            return;
        }
        for (ObjectiveGourp gourp : dungeonInstance.getObjectiveGourps()) {
            List<KillMobObjective> objectiveList = gourp.getObjectives(KillMobObjective.class);
            for (KillMobObjective killMobObjective : objectiveList) {
                if (killMobObjective.isValidEntity(dungeonInstance, entity)) {
                    killMobObjective.call();
                }
            }
        }
        dungeonInstance.checkObjectiveGroup();
    }

    /*
    @EventHandler(ignoreCancelled = true)
    public void onPlayerMove(PlayerMoveEvent event) {
        Player player = event.getPlayer();
        if (!isDungeonWorld(player.getWorld())) {
            return;
        }
        DungeonInstance dungeonInstance = DungeonManager.getDungeonInstance(player.getWorld());
        if (dungeonInstance == null || !dungeonInstance.getState().equals(DungeonState.RUNNING)) {
            return;
        }
        for (ObjectiveGourp gourp : dungeonInstance.getObjectiveGourps()) {
            List<LocationObjective> objectiveList1 = gourp.getObjectives(LocationObjective.class);
            if (objectiveList1.size() > 0) {
                for (LocationObjective locationObjective : objectiveList1) {
                    if (locationObjective.isArrive(dungeonInstance, player)) {
                        locationObjective.call();
                    }
                }
            }
            List<RegionObjective> objectiveList2 = gourp.getObjectives(RegionObjective.class);
            if (objectiveList2.size() > 0) {
                for (RegionObjective regionObjective : objectiveList2) {
                    if (regionObjective.isInRegion(dungeonInstance, player)) {
                        regionObjective.call();
                    }
                }
            }
        }
        dungeonInstance.checkObjectiveGroup();
    }
     */

    private boolean isDungeonWorld(World world) {
        return world.getName().startsWith("dw_");
    }

    private boolean isInDungeon(Player player) {
        return DungeonManager.getDungeonInstance(player) != null;
    }

}
