package com.io.yutian.thewardungeon.script;

import com.io.yutian.thewardungeon.data.LineMetadata;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.manager.RegistryManager;
import com.io.yutian.thewardungeon.util.ConsoleColor;
import com.io.yutian.thewardungeon.util.Log;
import com.io.yutian.thewardungeon.util.LogUtil;
import com.io.yutian.thewarskyblocklib.util.EnumUtil;
import com.io.yutian.thewarskyblocklib.util.StringUtil;
import javassist.bytecode.Opcode;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.Map;

public abstract class Script implements Cloneable {

    public LineMetadata metadata;

    public SelectorType selectorType;
    public Object trigger;

    public Script(LineMetadata metadata) {
        this.metadata = metadata;
    }

    public abstract void execute(DungeonInstance dungeonInstance);

    @Override
    public Script clone() {
        try {
            return (Script) super.clone();
        } catch (CloneNotSupportedException e) {
            return this;
        }
    }

    public static Script getScript(String arg) {
        String key;
        String s = arg.toLowerCase();
        if (arg.startsWith("script ")) {
            try {
                String scriptId = arg.split(" ")[1];
                return new NestScript(null, scriptId);
            } catch (Exception e) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: Nest" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + arg);
                return null;
            }
        } else if (arg.startsWith("delay ")) {
            try {
                String time = arg.split(" ")[1];
                if (!StringUtil.isInt(time)) {
                    Log.error(ConsoleColor.RED + "加载脚本时发生错误: Delay" + ConsoleColor.WHITE);
                    Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + arg);
                    Log.error(ConsoleColor.RED + "[Cause]: " + ConsoleColor.WHITE + time+ " 不是数字");
                    return null;
                }
                int i = Integer.parseInt(time);
                return new DelayScript(null, i);
            } catch (Exception e) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: Delay" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + arg);
                return null;
            }
        }
        if (s.contains("{")) {
            key = s.substring(0, s.indexOf("{"));
        } else if (s.contains("[")) {
            key = s.substring(0, s.indexOf("["));
        } else {
            key = s;
        }
        Map<String, String> valueMap = new HashMap<>();
        String s2 = arg;
        SelectorType selectorType1 = SelectorType.NULL;
        if (s2.contains("{") && s2.contains("}")) {
            int count = 0;
            String s3 = s2.substring(s2.indexOf(Opcode.LSHR) + 1, s2.lastIndexOf(Opcode.LUSHR));
            char[] charArray = s3.toCharArray();
            int length = charArray.length;
            for (int i = 0; i < length; i++) {
                char c = charArray[i];
                count = c == '{' ? count + 1 : count;
                if (c == '}') {
                    count--;
                }
            }
            if (count != 0) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: 不平衡支撑" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + s3);
            } else if (s3.length() != 0) {
                int start = 0;
                int pos = 0;
                int depth = 0;
                String s4 = s3 + "}";
                char[] charArray2 = s4.toCharArray();
                int length2 = charArray2.length;
                for (int i2 = 0; i2 < length2; i2++) {
                    char c2 = charArray2[i2];
                    depth = (c2 == '}' || c2 == ']') ? ((c2 == '{' || c2 == '[') ? depth + 1 : depth) - 1 : depth;
                    if ((c2 == ';' && depth == 0) || (c2 == '}' && depth < 0)) {
                        try {
                            String element = s4.substring(start, pos);
                            if (pos - start > 0 && element.length() > 0) {
                                valueMap.put(element.substring(0, element.indexOf(61)).trim().toLowerCase(), element.substring(element.indexOf(61) + 1).trim());
                            }
                            start = pos + 1;
                        } catch (Exception e) {
                            start = pos + 1;
                        } catch (Throwable th) {
                            int i3 = pos + 1;
                            throw th;
                        }
                    }
                    pos++;
                }
                String s5 = s2.substring(key.length()+pos+1);
                if (!s5.isEmpty()) {
                    if (s5.contains(" @")) {
                        String arg1 = s5.split("@")[1];
                        if (!EnumUtil.isValidEnum(SelectorType.class, arg1.toUpperCase())) {
                            Log.error(ConsoleColor.RED + "加载脚本时发生错误: 未知的选择器类型" + ConsoleColor.WHITE);
                            Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + arg1);
                            return null;
                        }
                        selectorType1 = SelectorType.valueOf(arg1.toUpperCase());
                    }
                }
            }
        } else if (!s2.contains("[") || !s2.contains("]")) {
            key = s2;
        } else {
            try {
                s2 = s2.split("\\[")[1].split("\\]")[0];
            } catch (ArrayIndexOutOfBoundsException e2) {
                Log.error(ConsoleColor.RED + "加载脚本时发生错误: 无效语法" + ConsoleColor.WHITE);
                Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + s2);
            }
        }

        key = key.toLowerCase();
        Class<? extends Script> clazz = RegistryManager.getScriptRegistry().get(key);
        if (clazz == null) {
            Log.error(ConsoleColor.RED + "加载脚本时发生错误: 无效脚本" + ConsoleColor.WHITE);
            Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + key);
            return null;
        }
        try {
            Constructor constructor = clazz.getConstructor(LineMetadata.class);
            Script script = (Script) constructor.newInstance(new LineMetadata(arg, valueMap));
            script.selectorType = selectorType1;
            return script;
        } catch (Exception e) {
            Log.error(ConsoleColor.RED + "加载脚本时发生错误: 未知错误" + ConsoleColor.WHITE);
            Log.error(ConsoleColor.RED + "[Line]: " + ConsoleColor.WHITE + key);
            e.printStackTrace();
            return null;
        }
    }

    protected static class NestScript extends Script {

        private String group;

        public NestScript(LineMetadata metadata, String group) {
            super(metadata);
            this.group = group;
        }

        @Override
        public void execute(DungeonInstance dungeonInstance) {
            ScriptGourp gourp = dungeonInstance.getDungeon().getConfig().getScriptGourp(group);
            if (gourp == null) {
                LogUtil.logScript(dungeonInstance, metadata, NestScript.class, "未知的脚本组");
                return;
            }
            gourp.execute(dungeonInstance);
        }

    }

    protected static class DelayScript extends Script {

        public int time;

        public DelayScript(LineMetadata metadata, int time) {
            super(metadata);
            this.time = time;
        }

        @Override
        public void execute(DungeonInstance dungeonInstance) {
        }

    }

    @Override
    public String toString() {
        return this.getClass().getSimpleName()+"{"+ metadata +'}';
    }
}
