package com.io.yutian.thewardungeon.script.list;

import com.io.yutian.thewardungeon.data.LineMetadata;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.script.Script;
import com.io.yutian.thewardungeon.util.LineMetadataUtil;
import com.io.yutian.thewardungeon.util.LogUtil;
import com.io.yutian.thewarskyblocklib.math.Point;
import com.io.yutian.thewarskyblocklib.util.EnumUtil;
import com.io.yutian.thewarskyblocklib.util.MathUtil;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.block.Block;

public class FillBlockScript extends Script {

    private String type;
    private Point start;
    private Point end;

    public FillBlockScript(LineMetadata metadata) {
        super(metadata);
        this.type = metadata.getString("type", "STONE").toUpperCase();
        this.start = LineMetadataUtil.readPoint(metadata, "start", null);
        this.end = LineMetadataUtil.readPoint(metadata, "end", null);
    }

    @Override
    public void execute(DungeonInstance dungeonInstance) {
        if (start == null) {
            LogUtil.logScript(dungeonInstance, metadata, FillBlockScript.class, "开始坐标参数错误");
            return;
        }
        if (end == null) {
            LogUtil.logScript(dungeonInstance, metadata, FillBlockScript.class, "结束坐标参数错误");
            return;
        }
        if (!EnumUtil.isValidEnum(Material.class, type)) {
            LogUtil.logScript(dungeonInstance, metadata, FillBlockScript.class, "方块类型参数错误");
            return;
        }
        int minX = (int) MathUtil.round(Math.min(start.getX(), end.getX()), 0);
        int minY = (int) MathUtil.round(Math.min(start.getY(), end.getY()), 0);
        int minZ = (int) MathUtil.round(Math.min(start.getZ(), end.getZ()), 0);
        int maxX = (int) MathUtil.round(Math.max(start.getX(), end.getX()), 0);
        int maxY = (int) MathUtil.round(Math.max(start.getY(), end.getY()), 0);
        int maxZ = (int) MathUtil.round(Math.max(start.getZ(), end.getZ()), 0);
        World world = dungeonInstance.getWorldInstance().getWorld();
        Material material = Material.valueOf(type);
        for (int x = minX; x <= maxX; x++) {
            for (int y = minY; y <= maxY; y++) {
                for (int z = minZ; z <= maxZ; z++) {
                    Block block = world.getBlockAt(x, y, z);
                    block.setType(material);
                }
            }
        }
    }

}