package com.io.yutian.thewardungeon.script.list;

import com.io.yutian.thewardungeon.data.LineMetadata;
import com.io.yutian.thewardungeon.dungeon.DungeonInstance;
import com.io.yutian.thewardungeon.script.Script;
import com.io.yutian.thewardungeon.util.LineMetadataUtil;
import com.io.yutian.thewardungeon.util.LogUtil;
import com.io.yutian.thewarskyblocklib.math.Point;
import io.lumine.mythic.api.mobs.MythicMob;
import io.lumine.mythic.bukkit.BukkitAdapter;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.core.mobs.ActiveMob;
import org.bukkit.Location;
import org.bukkit.entity.EntityType;

import java.util.Optional;

public class SpawnMobScript extends Script {

    private String type;
    private int amount;
    private Point location;

    public SpawnMobScript(LineMetadata metadata) {
        super(metadata);
        this.type = metadata.getString("type", "minecraft:zombie");
        this.amount = metadata.getInt("amount", 1);
        this.location = LineMetadataUtil.readPoint(metadata, "location", null);
    }

    @Override
    public void execute(DungeonInstance dungeonInstance) {
        if (!type.contains(":") || type.startsWith(":") || type.endsWith(":")) {
            LogUtil.logScript(dungeonInstance, metadata, SpawnMobScript.class, "实体类型格式错误");
            return;
        }
        if (location == null) {
            LogUtil.logScript(dungeonInstance, metadata, SpawnMobScript.class, "坐标参数错误");
            return;
        }
        String prefix = type.split(":")[0].toLowerCase();
        String type1 = type.split(":")[1];
        Location location1 = location.toLocation(dungeonInstance.getWorldInstance().getWorld());
        if (prefix.equalsIgnoreCase("minecraft")) {
            try {
                EntityType entityType = EntityType.valueOf(type1.toUpperCase());
                for (int i = 0; i < amount; i++) {
                    dungeonInstance.getWorldInstance().getWorld().spawnEntity(location1, entityType);
                }
            } catch (Exception e) {
                LogUtil.logScript(dungeonInstance, metadata, SpawnMobScript.class, "未知的实体类型");
            }
        } else if (prefix.equalsIgnoreCase("mm") || prefix.equalsIgnoreCase("mythicmob")) {
            Optional<MythicMob> optional = MythicBukkit.inst().getMobManager().getMythicMob(type1);
            if (!optional.isPresent()) {
                LogUtil.logScript(dungeonInstance, metadata, SpawnMobScript.class, "未知的实体类型");
                return;
            }
            MythicMob mob = optional.get();
            for (int i = 0; i < amount; i++) {
                ActiveMob activeMob = mob.spawn(BukkitAdapter.adapt(location1), 0.0);
                dungeonInstance.addActiveMob(activeMob);
            }
        } else {
            LogUtil.logScript(dungeonInstance, metadata, SpawnMobScript.class, "未知的实体类型");
        }
    }

}
