package com.io.yutian.thewarskyblocklib.command.list;

import com.io.yutian.thewardungeon.lang.Lang;
import com.io.yutian.thewarskyblocklib.command.*;
import com.io.yutian.thewarskyblocklib.command.argument.Argument;
import com.io.yutian.thewarskyblocklib.command.argument.ArgumentTypes;
import com.io.yutian.thewarskyblocklib.util.PageList;
import com.io.yutian.thewarskyblocklib.util.TextComponentBuilder;
import net.md_5.bungee.api.chat.ClickEvent;
import net.md_5.bungee.api.chat.HoverEvent;
import net.md_5.bungee.api.chat.hover.content.Text;
import org.bukkit.command.CommandSender;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class CommandHelp extends ICommand {

    private Command parent;
    private String helpTitle;

    public CommandHelp(Command parent, String helpTitle) {
        super("help");
        this.parent = parent;
        this.helpTitle = helpTitle;
        addArgument(Argument.argument("page", ArgumentTypes.INTEGER).optional(1));
    }

    @Override
    public boolean hasPermission(CommandSender sender) {
        return true;
    }

    @Override
    public void executes(CommandContext commandContext) {
        String label = commandContext.getCommandLabel();
        CommandSender sender = commandContext.getSender();
        int page = commandContext.getArgumentsValue("page").getInt();
        if (page <= 0) {
            sender.sendMessage(Lang.prefix+ Lang.get("command.help.page-error"));
            return;
        }
        List<ICommand> commands = parent.getCommands();
        Stream<ICommand> stream = commands.stream().filter((c) -> c.hasPermission(sender) && !c.hide());
        List<ICommand> list = stream.collect(Collectors.toList());
        PageList<ICommand> pageList = new PageList<>(list, 8);
        if (page > pageList.size()) {
            sender.sendMessage(Lang.prefix+ Lang.get("command.help.page-error"));
            return;
        }
        sender.sendMessage(" ");
        List<ICommand> commandList = pageList.getList(page);
        sender.sendMessage("§7======[ §e§l"+helpTitle+" §7]======");
        for (ICommand command : commandList) {
            StringBuilder stringBuilder = new StringBuilder("§6/"+label+" "+command.getName());
            stringBuilder.append("§f");
            if (command.getCommandNodes().size() > 0) {
                StringBuilder sb = new StringBuilder();
                sb.append(" [");
                int i = 0;
                for (CommandNode node : command.getCommandNodes()) {
                    sb.append(node.getName());
                    if (i + 1 < command.getCommandNodes().size()) {
                        sb.append("/");
                    }
                    i++;
                }
                sb.append("]");
                stringBuilder.append(sb);
            } else {
                for (Argument argument : command.getArguments()) {
                    stringBuilder.append(" ");
                    stringBuilder.append("<"+argument.getName()+">");
                }
            }
            if (command.getDescription() != null) {
                stringBuilder.append(" ");
                stringBuilder.append("§7- §f"+command.getDescription());
            }
            sender.sendMessage(stringBuilder.toString());
        }
        TextComponentBuilder builder = new TextComponentBuilder();
        boolean hasUpPage = page > 1;
        boolean hasNextPage = page < pageList.size();
        builder.addText("§7=====");
        if (hasUpPage) {
            builder.addText(" §7["+getColor(true)+"◀§7] ", new ClickEvent(ClickEvent.Action.RUN_COMMAND, "/"+label+" help "+(page-1)), new HoverEvent(HoverEvent.Action.SHOW_TEXT, Arrays.asList(new Text("§f上一頁"))));
        } else {
            builder.addText(" §7["+getColor(false)+"◀§7] ");
        }
        builder.addText("§7====");
        builder.addText("(§a"+page+"§f/§e"+pageList.size()+"§7)");
        builder.addText("§7====");
        if (hasNextPage) {
            builder.addText(" §7["+getColor(true)+"▶§7] ", new ClickEvent(ClickEvent.Action.RUN_COMMAND, "/"+label+" help "+(page+1)), new HoverEvent(HoverEvent.Action.SHOW_TEXT, Arrays.asList(new Text("§f下一頁"))));
        } else {
            builder.addText(" §7["+getColor(false)+"▶§7] ");
        }
        builder.addText("§7=====");
        sender.spigot().sendMessage(builder.build());
    }

    private String getColor(boolean hasPage) {
        return hasPage ? "§a" : "§c";
    }

}
