package com.io.yutian.thewarskyblocklib.config;

import com.io.yutian.thewardungeon.util.Log;
import com.io.yutian.thewarskyblocklib.serializa.ConfigSerializable;
import com.io.yutian.thewarskyblocklib.serializa.SerializaUtil;
import com.io.yutian.thewarskyblocklib.util.Logger;
import org.bukkit.Location;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class Config {

    private File file;
    private FileConfiguration fileConfiguration;
    private Class<?> configClass;

    private Map<String, ConfigValue> registerMap = new HashMap<>();
    private Map<String, String> fieldMap = new HashMap<>();

    public Config(File file, Class<?> configClass) {
        this.file = file;
        this.configClass = configClass;
    }

    public void define(String fieldName, String path, ConfigValue configValue) {
        try {
            Field field = configClass.getDeclaredField(fieldName);
            if (field == null) {
                Logger.warning("TheWarSkyBlockLib","加载配置项时出现错误 原因: 目标成员变量不存在");
                return;
            }
            field.setAccessible(true);
            field.set(null, configValue);
        } catch (Exception e) {
            e.printStackTrace();
            return;
        }

        fieldMap.put(path, fieldName);
        registerMap.put(path, configValue);
    }

    public boolean load() {
        fileConfiguration = YamlConfiguration.loadConfiguration(file);
        try {
            for (Map.Entry<String, ConfigValue> entry : registerMap.entrySet()) {
                ConfigValue configValue = registerMap.get(entry.getKey());
                Object obj = configValue.load(entry.getKey(), fileConfiguration);
                String fieldName = fieldMap.get(entry.getKey());
                Field field = configClass.getDeclaredField(fieldName);
                if (field == null) {
                    Log.warning("加载配置项时出现错误 原因: 目标成员变量不存在");
                    continue;
                }
                configValue.setValue(obj);

                Class<?> configValueClass = ConfigValue.class;
                Field field1 = configValueClass.getDeclaredField("file");
                field1.setAccessible(true);
                field1.set(configValue, file);
                Field field2 = configValueClass.getDeclaredField("configuration");
                field2.setAccessible(true);
                field2.set(configValue, fileConfiguration);
                Field field3 = configValueClass.getDeclaredField("path");
                field3.setAccessible(true);
                field3.set(configValue, entry.getKey());
                Field field4 = configValueClass.getDeclaredField("field");
                field4.setAccessible(true);
                field4.set(configValue, field);
                field.setAccessible(true);
                field.set(null, configValue);
            }
            return true;
        } catch (Exception e) {
            Log.warning("加载配置项时出现错误 原因: 未知");
            e.printStackTrace();
            return false;
        }
    }

    public static class StringValue extends ConfigValue<String> {
        @Override
        public String load(String path, FileConfiguration config) {
            return ConfigReader.STRING.read(path, configuration);
        }

        @Override
        public String getDefault() {
            return "null";
        }
    }

    public static class BooleanValue extends ConfigValue<Boolean> {
        @Override
        public Boolean load(String path, FileConfiguration config) {
            return ConfigReader.BOOLEAN.read(path, configuration);
        }

        @Override
        public Boolean getDefault() {
            return false;
        }
    }

    public static class IntValue extends ConfigValue<Integer> {
        @Override
        public Integer load(String path, FileConfiguration config) {
            return ConfigReader.INTEGER.read(path, configuration);
        }

        @Override
        public Integer getDefault() {
            return 0;
        }
    }

    public static class DoubleValue extends ConfigValue<Double> {
        @Override
        public Double load(String path, FileConfiguration config) {
            return ConfigReader.DOUBLE.read(path, configuration);
        }

        @Override
        public Double getDefault() {
            return 0d;
        }
    }

    public static class FloatValue extends ConfigValue<Float> {
        @Override
        public Float load(String path, FileConfiguration config) {
            return ConfigReader.FLOAT.read(path, configuration);
        }

        @Override
        public Float getDefault() {
            return 0f;
        }
    }

    public static class LongValue extends ConfigValue<Long> {
        @Override
        public Long load(String path, FileConfiguration config) {
            return ConfigReader.LONG.read(path, config);
        }

        @Override
        public Long getDefault() {
            return 0L;
        }

    }

    public static class EnumValue<T extends Enum> extends ConfigValue<T> {

        protected Class<T> enumType;

        public EnumValue(Class<T> enumType) {
            this.enumType = enumType;
        }

        @Override
        public T load(String path, FileConfiguration config) {
            String string = config.getString(path);
            return (T) Enum.valueOf(enumType, string);
        }

        @Override
        public T getDefault() {
            return null;
        }

    }

    public static class ListValue<V> extends ConfigValue<List<V>> {
        @Override
        public List<V> load(String path, FileConfiguration config) {
            return (List<V>) ConfigReader.LIST.read(path, config);
        }

        @Override
        public List<V> getDefault() {
            return new ArrayList<>();
        }
    }

    public static class StringListValue extends ListValue<String> {
        @Override
        public List<String> load(String path, FileConfiguration config) {
            return ConfigReader.STRING_LIST.read(path, config);
        }

        @Override
        public List<String> getDefault() {
            return new ArrayList<>();
        }
    }

    public static class LocationValue extends ConfigValue<Location> {
        @Override
        public Location load(String path, FileConfiguration config) {
            return ConfigReader.LOCATION.read(path, config);
        }

        @Override
        public Location getDefault() {
            return null;
        }
    }

    public static class SerializaValue<J extends ConfigSerializable> extends ConfigValue<J> {

        protected Class<J> typeClass;

        public SerializaValue(Class<J> typeClass) {
            this.typeClass = typeClass;
        }

        @Override
        public J load(String path, FileConfiguration config) {
            ConfigurationSection configurationSection = config.getConfigurationSection(path);
            return SerializaUtil.deserialize(typeClass, configurationSection);
        }

        @Override
        public void save(String path, FileConfiguration configuration, J value) {
            Map<String, Object> map = value.serialize();
            for (Map.Entry<String, Object> entry : map.entrySet()) {
                configuration.set(path+"."+entry.getKey(), entry.getValue());
            }
        }

        @Override
        public J getDefault() {
            return null;
        }
    }

    public static class ObjectValue<K> extends ConfigValue<K> {

        private ConfigObjectDeserializable<K> configObjectDeserializable;

        public ObjectValue(ConfigObjectDeserializable<K> configObjectDeserializable) {
            this.configObjectDeserializable = configObjectDeserializable;
        }

        @Override
        public K load(String path, FileConfiguration config) {
            return configObjectDeserializable.load(path, config);
        }

        @Override
        public void save(String path, FileConfiguration configuration, K value) {
            this.configObjectDeserializable.save(path, configuration, value);
        }

        @Override
        public K getDefault() {
            return null;
        }
    }

    public static abstract class ConfigValue<T> {

        private T value;

        protected File file;
        protected FileConfiguration configuration;
        protected String path;
        protected Field field;

        public abstract T load(String path, FileConfiguration config);

        public abstract T getDefault();

        public void save(String path, FileConfiguration configuration, T value) {
            configuration.set(path, value);
        }

        public void setValue(T value) {
            this.value = value;
        }

        public void save() {
            this.save(path, configuration, value);
            try {
                configuration.save(file);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        @Nullable
        public T get() {
            return value;
        }

    }

    public interface ConfigObjectDeserializable<K> {

        K load(String path, FileConfiguration config);
        void save(String path, FileConfiguration config, K value);

    }


}
