package com.io.yutian.thewarskyblocklib.manager;

import com.io.yutian.thewarskyblocklib.command.Command;
import com.io.yutian.thewarskyblocklib.command.SimpleCommandHandler;
import com.io.yutian.thewarskyblocklib.util.Logger;
import org.bukkit.Bukkit;
import org.bukkit.command.CommandMap;
import org.bukkit.plugin.Plugin;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CommandManager {

    private static final List<Command> commands = new ArrayList<>();

    protected static Map<String, org.bukkit.command.Command> bukkitCommandMap = new HashMap<>();
    protected static CommandMap commandMap;

    private static Command command;

    public static void register(Command command) {
        if (command.isFrozen() || commands.contains(command)) {
            return;
        }
        command.frozen();
        commands.add(command);
        if (command.getAliases().size() > 0) {
            for (String name : command.getAliases()) {
                registerBukkitCommand(command.getPlugin(), name, command);
            }
        }
        registerBukkitCommand(command.getPlugin(), command.getCommand(), command);
    }

    public static void unregisterAll() {
        for (Command command : commands) {
            unregister(command);
        }
        commands.clear();
    }

    public static void unregister(Command command) {
        try {
            Map<String, Command> map = (Map<String, Command>) commandMap.getClass().getMethod("getKnownCommands").invoke(commandMap);
            if (command.getAliases().size() > 0) {
                for (String arg : command.getAliases()) {
                    map.remove(arg);
                    org.bukkit.command.Command bukkitCommand = bukkitCommandMap.get(arg);
                    bukkitCommand.unregister(commandMap);
                }
            }
            commandMap.getCommand(command.getCommand()).unregister(commandMap);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    protected static void registerBukkitCommand(Plugin plugin, String name, Command command) {
        SimpleCommandHandler simpleCommandHandler = new SimpleCommandHandler(name, command);
        bukkitCommandMap.put(name, simpleCommandHandler);
        commandMap.register(plugin.getName(), simpleCommandHandler);
    }

    public static List<Command> getCommands() {
        return commands;
    }

    static {
        try {
            Class<?> c = Bukkit.getServer().getClass();
            for (Method method : c.getDeclaredMethods()) {
                if (method.getName().equals("getCommandMap")) {
                    commandMap = (CommandMap) method.invoke(Bukkit.getServer(), new Object[0]);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            Logger.error("TheWarSkyBlockLib", "未能正常初始化命令");

        }
    }

}
