package com.io.yutian.thewarskyblocklib.nms.nbt;

import net.minecraft.nbt.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class NBTBase {

    public abstract Object getValue();

    public abstract byte getTypeId();

    protected static String getNBTTag(byte typeByte) {
        return switch (typeByte) {
            case 1 -> "NBTTagByte";
            case 2 -> "NBTTagShort";
            case 3 -> "NBTTagInt";
            case 4 -> "NBTTagLong";
            case 5 -> "NBTTagFloat";
            case 6 -> "NBTTagDouble";
            case 7 -> "NBTTagByteArray";
            case 8 -> "NBTTagString";
            case 9 -> "NBTTagList";
            case 10 -> "NBTTagCompound";
            case 11 -> "NBTTagIntArray";
            case 12 -> "NBTTagLongArray";
            default -> null;
        };
    }

    public static NBTBase asObject(Object object) {
        if (object instanceof Byte b) {
            return new NBTByte(b);
        } else if (object instanceof Short value) {
            return new NBTShort(value);
        } else if (object instanceof Integer value) {
            return new NBTInt(value);
        } else if (object instanceof Long value) {
            return new NBTLong(value);
        } else if (object instanceof Float value) {
            return new NBTFloat(value);
        } else if (object instanceof Double value) {
            return new NBTDouble(value);
        } else if (object instanceof byte[] value) {
            return new NBTByteArray(value);
        } else if (object instanceof String value) {
            return new NBTString(value);
        } else if (object instanceof int[] value) {
            return new NBTIntArray(value);
        } else if (object instanceof long[] value) {
            return new NBTLongArray(value);
        } else if (object instanceof Boolean b) {
            return new NBTByte((byte) (b ? 1 : 0));
        }
        return null;
    }

    public static net.minecraft.nbt.NBTBase asNMS(NBTBase nbtBase) {
        byte type = nbtBase.getTypeId();
        switch (type) {
            case 1 -> {
                NBTByte nbtByte = (NBTByte) nbtBase;
                return NBTTagByte.a(nbtByte.getByte());
            }
            case 2 -> {
                NBTShort nbtShort = (NBTShort) nbtBase;
                return NBTTagShort.a(nbtShort.getShort());
            }
            case 3 -> {
                NBTInt nbtInt = (NBTInt) nbtBase;
                return NBTTagInt.a(nbtInt.getInt());
            }
            case 4 -> {
                NBTLong nbtLong = (NBTLong) nbtBase;
                return NBTTagLong.a(nbtLong.getLong());
            }
            case 5 -> {
                NBTFloat nbtfloat = (NBTFloat) nbtBase;
                return NBTTagFloat.a(nbtfloat.getFloat());
            }
            case 6 -> {
                NBTDouble nbtDouble = (NBTDouble) nbtBase;
                return NBTTagDouble.a(nbtDouble.getDouble());
            }
            case 7 -> {
                NBTByteArray nbtByteArray = (NBTByteArray) nbtBase;
                return new NBTTagByteArray(nbtByteArray.getByteArray());
            }
            case 8 -> {
                NBTString nbtString = (NBTString) nbtBase;
                return NBTTagString.a(nbtString.getString());
            }
            case 9 -> {
                NBTList nbtTagList = (NBTList) nbtBase;
                List<net.minecraft.nbt.NBTBase> list = new ArrayList<>();
                for (Object base : nbtTagList.getList()) {
                    list.add(asNMS((NBTBase) base));
                }
                NBTTagList nbtTagList1 = new NBTTagList();
                for (net.minecraft.nbt.NBTBase nbtBase1 : list) {
                    nbtTagList1.add(nbtBase1);
                }
                return nbtTagList1;
            }
            case 10 -> {
                NBTCompound nbtCompound = (NBTCompound) nbtBase;
                NBTTagCompound nbtTagCompound = new NBTTagCompound();
                for (String key : nbtCompound.keySet()) {
                    NBTBase nbtBase1 = nbtCompound.get(key);
                    nbtTagCompound.a(key, asNMS(nbtBase1));
                }
                return nbtTagCompound;
            }
            case 11 -> {
                NBTIntArray nbtIntArray = (NBTIntArray) nbtBase;
                return new NBTTagIntArray(nbtIntArray.getIntArray());
            }
            case 12 -> {
                NBTLongArray nbtLongArray = (NBTLongArray) nbtBase;
                return new NBTTagLongArray(nbtLongArray.getLongArray());
            }
        }
        return null;
    }

    public static NBTBase as(net.minecraft.nbt.NBTBase nbtBase) {
        byte type = nbtBase.a();
        switch (type) {
            case 1 -> {
                NBTTagByte nbtTagByte = (NBTTagByte) nbtBase;
                return new NBTByte(nbtTagByte.a());
            }
            case 2 -> {
                NBTTagShort nbtTagShort = (NBTTagShort) nbtBase;
                return new NBTShort(nbtTagShort.h());
            }
            case 3 -> {
                NBTTagInt nbtTagInt = (NBTTagInt) nbtBase;
                return new NBTInt(nbtTagInt.g());
            }
            case 4 -> {
                NBTTagLong nbtTagLong = (NBTTagLong) nbtBase;
                return new NBTLong(nbtTagLong.f());
            }
            case 5 -> {
                NBTTagFloat nbtTagFloat = (NBTTagFloat) nbtBase;
                return new NBTFloat(nbtTagFloat.j());
            }
            case 6 -> {
                NBTTagDouble nbtTagDouble = (NBTTagDouble) nbtBase;
                return new NBTDouble(nbtTagDouble.i());
            }
            case 7 -> {
                NBTTagByteArray tagByteArray = (NBTTagByteArray) nbtBase;
                return new NBTByteArray(tagByteArray.d());
            }
            case 8 -> {
                NBTTagString nbtTagString = (NBTTagString) nbtBase;
                return new NBTString(nbtTagString.e_());
            }
            case 9 -> {
                NBTTagList nbtTagList = (NBTTagList) nbtBase;
                List<NBTBase> list = new ArrayList<>();
                for (net.minecraft.nbt.NBTBase base : nbtTagList) {
                    list.add(as(base));
                }
                return new NBTList(list);
            }
            case 10 -> {
                NBTTagCompound nbtTagCompound = (NBTTagCompound) nbtBase;
                Map<String, NBTBase> map = new HashMap<>();
                for (String key : nbtTagCompound.d()) {
                    map.put(key, as(nbtTagCompound.c(key)));
                }
                return new NBTCompound(map);
            }
            case 11 -> {
                NBTTagIntArray nbtTagIntArray = (NBTTagIntArray) nbtBase;
                return new NBTIntArray(nbtTagIntArray.f());
            }
            case 12 -> {
                NBTTagLongArray nbtTagLongArray = (NBTTagLongArray) nbtBase;
                return new NBTLongArray(nbtTagLongArray.f());
            }
        }
        return null;
    }

}
