package com.io.yutian.thewarskyblocklib.nms.nbt;

import net.minecraft.nbt.GameProfileSerializer;
import net.minecraft.nbt.NBTTagCompound;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

public class NBTCompound extends NBTBase {

    public static byte TYPE_ID = 10;

    private Map<String, NBTBase> nbtMap = new HashMap<>();

    public NBTCompound() {
    }

    public NBTCompound(Map<String, NBTBase> nbtMap) {
        this.nbtMap = nbtMap;
    }

    public Set<String> keySet() {
        return nbtMap.keySet();
    }

    public Map<String, NBTBase> getNBTMap() {
        return nbtMap;
    }

    public void remove(String key) {
        nbtMap.remove(key);
    }

    public boolean hasKey(String key) {
        return this.nbtMap.containsKey(key);
    }

    public boolean hasKey(String key, byte id) {
        return this.nbtMap.containsKey(key) && this.nbtMap.get(key).getTypeId() == id;
    }

    public boolean hasKey(String key, int id) {
        return this.nbtMap.containsKey(key) && this.nbtMap.get(key).getTypeId() == id;
    }

    public NBTList getList(String key) {
        NBTBase list = this.nbtMap.get(key);
        if (list == null || list.getTypeId() != 9) {
            return null;
        }
        return (NBTList) list;
    }

    public NBTCompound getCompound(String key) {
        NBTBase comound = this.nbtMap.get(key);
        if (comound == null || comound.getTypeId() != 10) {
            return null;
        }
        return (NBTCompound) comound;
    }

    public NBTBase get(String key) {
        return this.nbtMap.get(key);
    }

    public int getInt(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 3) {
            return 0;
        }
        return ((NBTInt) nbt).getInt();
    }

    public String getString(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 8) {
            return null;
        }
        return ((NBTString) nbt).getString();
    }

    public double getDouble(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 6) {
            return 0.0;
        }
        return ((NBTDouble) nbt).getDouble();
    }

    public float getFloat(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 5) {
            return 0.0F;
        }
        return ((NBTFloat) nbt).getFloat();
    }

    public short getShort(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 2) {
            return 0;
        }
        return ((NBTShort) nbt).getShort();
    }

    public long getLong(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 4) {
            return 0;
        }
        return ((NBTLong) nbt).getLong();
    }

    public byte getByte(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 1) {
            return 0;
        }
        return ((NBTByte) nbt).getByte();
    }

    public UUID getUUID(String key) {
        NBTBase nbt = this.nbtMap.get(key);
        if (nbt == null || nbt.getTypeId() != 11) {
            return null;
        }
        return GameProfileSerializer.a(NBTBase.asNMS((NBTIntArray) nbt));
    }

    public void put(String key, NBTBase nbt) {
        this.nbtMap.put(key, nbt);
    }

    public void putString(String key, String value) {
        put(key, new NBTString(value));
    }

    public void putInt(String key, int value) {
        put(key, new NBTInt(value));
    }

    public void putShort(String key, short value) {
        put(key, new NBTShort(value));
    }

    public void putDouble(String key, double value) {
        put(key, new NBTDouble(value));
    }

    public void putLong(String key, long value) {
        put(key, new NBTLong(value));
    }

    public void putFloat(String key, float value) {
        put(key, new NBTFloat(value));
    }

    public void putByte(String key, byte value) {
        put(key, new NBTByte(value));
    }

    public void putBoolean(String key, boolean value) {
        put(key, NBTByte.a(value));
    }

    public void putUUID(String key, UUID uuid) {
        put(key, new NBTIntArray(a(uuid)));
    }

    private static UUID a(int[] var0) {
        return new UUID((long)var0[0] << 32 | (long)var0[1] & 4294967295L, (long)var0[2] << 32 | (long)var0[3] & 4294967295L);
    }

    private static int[] a(UUID var0) {
        long var1 = var0.getMostSignificantBits();
        long var3 = var0.getLeastSignificantBits();
        return a(var1, var3);
    }

    private static int[] a(long var0, long var2) {
        return new int[]{(int)(var0 >> 32), (int)var0, (int)(var2 >> 32), (int)var2};
    }

    @Override
    public Object getValue() {
        return nbtMap;
    }

    @Override
    public byte getTypeId() {
        return TYPE_ID;
    }

    @Override
    public String toString() {
        return "NBTCompound{" +
                "nbtMap=" + nbtMap +
                '}';
    }

    public static NBTCompound asNMS(NBTTagCompound nbtTagCompound) {
        Map<String, NBTBase> map = new HashMap<>();
        for (String key : nbtTagCompound.d()) {
            net.minecraft.nbt.NBTBase nbtBase = nbtTagCompound.c(key);
            map.put(key, NBTBase.as(nbtBase));
        }
        return new NBTCompound(map);
    }

}
