package com.io.yutian.thewarskyblocklib.region;

import com.io.yutian.thewarskyblocklib.math.Point;
import com.io.yutian.thewarskyblocklib.serializa.ConfigSerializable;
import com.io.yutian.thewarskyblocklib.serializa.SerializaUtil;
import com.io.yutian.thewarskyblocklib.serializa.Serializable;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.configuration.ConfigurationSection;
import org.json.JSONObject;

import java.util.HashMap;
import java.util.Map;

public class Region implements Serializable<Region>, ConfigSerializable {

    private World world;
    private Point min;
    private Point max;

    public Region(World world, Point point1, Point point2) {
        this.world = world;
        this.min = new Point(Math.min(point1.getX(), point2.getX()), Math.min(point1.getY(), point2.getY()), Math.min(point1.getZ(), point2.getZ()));
        this.max = new Point(Math.max(point1.getX(), point2.getX()), Math.max(point1.getY(), point2.getY()), Math.max(point1.getZ(), point2.getZ()));
    }

    public World getWorld() {
        return world;
    }

    public Point getMin() {
        return min;
    }

    public Point getMax() {
        return max;
    }

    public boolean isInRegion(Location location) {
        if (!location.getWorld().equals(this.world)) {
            return false;
        }
        return (location.getBlockX() >= this.min.getX()
                && location.getBlockX() <= this.max.getX()
                && location.getBlockY() >= this.min.getY()
                && location.getBlockY() <= this.max.getY()
                && location.getBlockZ() >= this.min.getZ()
                && location.getBlockZ() <= this.max.getZ());
    }

    @Override
    public JSONObject toJson() {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("world", world.getName());
        jsonObject.put("min", min.toJson());
        jsonObject.put("max", max.toJson());
        return null;
    }

    public static Region of(JSONObject json) {
        World world1 = Bukkit.getWorld(json.getString("world"));
        Point min = SerializaUtil.deserialize(Point.class, json.getJSONObject("min"));
        Point max = SerializaUtil.deserialize(Point.class, json.getJSONObject("max"));
        return new Region(world1, min, max);
    }

    @Override
    public Map<String, Object> serialize() {
        Map<String, Object> map = new HashMap<>();
        map.put("world", world.getName());
        map.put("min", min.serialize());
        map.put("max", max.serialize());
        return map;
    }

    public static Region deserialize(ConfigurationSection section) {
        World world1 = Bukkit.getWorld(section.getString("world"));
        return new Region(world1, SerializaUtil.deserialize(Point.class, section.getConfigurationSection("min")), SerializaUtil.deserialize(Point.class, section.getConfigurationSection("max")));
    }

}
