package com.io.yutian.thewarskyblocklib.registry;

import com.google.common.collect.BiMap;
import com.google.common.collect.HashBiMap;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Stream;

public class Registry<K, E> implements Iterable<E> {
    protected BiMap<K, E> elements = HashBiMap.create();

    public E get(K key) {
        return this.elements.get(key);
    }

    public E getFirstIf(Predicate<E> predicate) {
        for (E element : this.elements.values()) {
            if (predicate.test(element)) {
                return element;
            }
        }
        return null;
    }

    public Collection<E> getAllIf(Predicate<E> predicate) {
        Collection<E> checked = new ArrayList<>();
        for (E element : this.elements.values()) {
            if (predicate.test(element)) {
                checked.add(element);
            }
        }
        return checked;
    }

    public Collection<E> getAll() {
        return new HashSet(this.elements.values());
    }

    public boolean containsKey(K key) {
        return this.elements.containsKey(key);
    }

    public boolean contains(E element) {
        return this.elements.containsValue(element);
    }

    public void add(K key, E element) {
        this.elements.put(key, element);
    }

    public void removeKey(K key) {
        this.elements.remove(key);
    }

    public void remove(E element) {
        this.elements.values().remove(element);
    }

    public Set<Map.Entry<K, E>> entrySet() {
        return this.elements.entrySet();
    }

    public Stream<E> stream() {
        return this.elements.values().stream();
    }

    @Override
    public Iterator<E> iterator() {
        return this.elements.values().iterator();
    }

    public int size() {
        return this.elements.size();
    }
}