package com.io.yutian.thewarskyblocklib.util;

import org.bukkit.plugin.Plugin;

import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

public class ClassUtil {

    public static boolean isAssignableFrom(Object object, Class clazz) {
        if (object == null) {
            return false;
        }
        Class<?> c = PRIMITIVE_TO_WRAPPER.getOrDefault(clazz, clazz);
        if (c.isAssignableFrom(object.getClass())) {
            return true;
        }
        if (object.getClass().equals(Integer.class)) {
            if (c.equals(Double.class) || c.equals(Float.class) || c.equals(Long.class)) {
                return true;
            }
        }
        if (object.getClass().equals(String.class)) {
            String s = (String) object;
            if (c.equals(Double.class)) {
                return StringUtil.isDouble(s);
            } else if (c.equals(Float.class)) {
                return StringUtil.isFloat(s);
            } else if (c.equals(Long.class)) {
                return StringUtil.isLong(s);
            }
        }
        return false;
    }

    public static List<String> getClassName(Plugin plugin, String packageName) {
        return getClassNameByJar(getJar(plugin), packageName, true);
    }

    public static List<String> getClassNameByJar(String jarPath, String packagePath, boolean childPackage) {
        List<String> classNames = new ArrayList<>();
        packagePath = packagePath.replace(".", "/");
        try {
            JarFile jarFile = new JarFile(jarPath);
            Enumeration<JarEntry> entrys = jarFile.entries();
            while (entrys.hasMoreElements()) {
                JarEntry jarEntry = entrys.nextElement();
                String entryName = jarEntry.getName();
                if (entryName.endsWith(".class")) {
                    if (childPackage) {
                        if (entryName.startsWith(packagePath)) {
                            entryName = entryName.replace("/", ".").substring(0, entryName.lastIndexOf("."));
                            classNames.add(entryName);
                        }
                    } else {
                        int index = entryName.lastIndexOf("/");
                        String myPackagePath;
                        if (index != -1) {
                            myPackagePath = entryName.substring(0, index);
                        } else {
                            myPackagePath = entryName;
                        }
                        if (myPackagePath.equals(packagePath)) {
                            entryName = entryName.replace("/", ".").substring(0, entryName.lastIndexOf("."));
                            classNames.add(entryName);
                        }
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return classNames;
    }

    public static List<String> getAllClassNameByJar(String jarPath) {
        List<String> classNames = new ArrayList<>();
        try {
            JarFile jarFile = new JarFile(jarPath);
            Enumeration<JarEntry> entrys = jarFile.entries();
            while (entrys.hasMoreElements()) {
                JarEntry jarEntry = entrys.nextElement();
                String entryName = jarEntry.getName();
                if (entryName.endsWith(".class")) {
                    entryName = entryName.replace("/", ".").substring(0, entryName.lastIndexOf("."));
                    classNames.add(entryName);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return classNames;
    }

    public static String getJar(Plugin plugin) {
        try {
            Class clazz = Class.forName(plugin.getDescription().getMain());
            String p = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
            return p.substring(1);
        } catch (Exception e) {
            return null;
        }
    }

    private static final Map<Class<?>, Class<?>> PRIMITIVE_TO_WRAPPER = new HashMap<>();

    static {
        PRIMITIVE_TO_WRAPPER.put(boolean.class, Boolean.class);
        PRIMITIVE_TO_WRAPPER.put(byte.class, Byte.class);
        PRIMITIVE_TO_WRAPPER.put(short.class, Short.class);
        PRIMITIVE_TO_WRAPPER.put(char.class, Character.class);
        PRIMITIVE_TO_WRAPPER.put(int.class, Integer.class);
        PRIMITIVE_TO_WRAPPER.put(long.class, Long.class);
        PRIMITIVE_TO_WRAPPER.put(float.class, Float.class);
        PRIMITIVE_TO_WRAPPER.put(double.class, Double.class);
    }

}
