package com.io.yutian.thewarskyblocklib.util;

import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class EnumUtil {

    public static <E extends Enum<E>> boolean isValidEnum(@Nullable Class<E> enumClass, @Nullable String valueName) {
        return getEnum(enumClass, valueName) != null;
    }

    public static <E extends Enum<E>> E getEnumIgnoreCase(@Nullable Class<E> enumClass, @Nullable String valueName, @Nullable E defaultValue) {
        return (E) getEnum(enumClass, valueName.toUpperCase(), defaultValue);
    }

    public static <E extends Enum<E>> E getEnumIgnoreCase(@Nullable Class<E> enumClass, @Nullable String valueName) {
        return (E) getEnumIgnoreCase(enumClass, valueName, null);
    }

    public static <E extends Enum<E>> E getEnum(@Nullable Class<E> enumClass, @Nullable String valueName, @Nullable E defaultValue) {
        if (enumClass == null || valueName == null) {
            return defaultValue;
        }
        try {
            return (E) Enum.valueOf(enumClass, valueName);
        } catch (IllegalArgumentException e) {
            return defaultValue;
        }
    }

    public static <E extends Enum<E>> E getEnum(@Nullable Class<E> enumClass, @Nullable String valueName) {
        return (E) getEnum(enumClass, valueName, null);
    }

    public static String getConvertedName(@NotNull Enum<?> anEnum) {
        char[] charArray = anEnum.name().replace('_', ' ').toLowerCase().toCharArray();
        boolean foundSpace = true;
        for (int i = 0; i < charArray.length; i++) {
            if (!Character.isLetter(charArray[i])) {
                foundSpace = true;
            } else if (foundSpace) {
                charArray[i] = Character.toUpperCase(charArray[i]);
                foundSpace = false;
            }
        }
        return String.valueOf(charArray);
    }


}
