package com.io.yutian.thewarskyblocklib.util;

import org.bukkit.plugin.Plugin;

import javax.annotation.Nullable;
import java.io.*;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;

public class FileUtil {

    public static List<File> getFilesForFolder(File folder) {
        List<File> files = new ArrayList<>();
        if (!folder.isDirectory()) {
            throw new IllegalArgumentException("File \"" + folder.getName() + "\" is not a directory");
        }
        File[] listFiles = folder.listFiles();
        for (File file : listFiles) {
            if (file.isDirectory()) {
                files.addAll(getFilesForFolder(file));
            } else {
                files.add(file);
            }
        }
        return files;
    }

    public static List<File> getSubFolders(File folder) {
        List<File> subFolders = new ArrayList<>();
        if (!folder.isDirectory()) {
            throw new IllegalArgumentException("File \"" + folder.getName() + "\" is not a directory");
        }
        File[] listFiles = folder.listFiles();
        for (File file : listFiles) {
            if (file.isDirectory()) {
                subFolders.add(file);
            }
        }
        return subFolders;
    }

    public static void copy(File inFile, File outFile) {
        if (inFile.isDirectory()) {
            copyDir(inFile, outFile, new String[0]);
        } else {
            copyFile(inFile, outFile);
        }
    }

    public static boolean copyFile(File inFile, File outFile) {
        return org.bukkit.util.FileUtil.copy(inFile, outFile);
    }

    public static void copyDir(File inDir, File outDir, String... exclude) {
        if (!outDir.exists()) {
            outDir.mkdir();
        }
        String[] list = inDir.list();
        for (String p : list) {
            if (!Arrays.asList(exclude).contains(p)) {
                copy(new File(inDir, p), new File(outDir, p));
            }
        }
    }

    public static boolean removeDir(File dir) {
        if (dir.isDirectory()) {
            for (File f : dir.listFiles()) {
                if (!removeDir(f)) {
                    return false;
                }
            }
        }
        return dir.delete();
    }


    public static @Nullable FileOutputStream getOutputStream(File file) {
        try {
            return new FileOutputStream(file);
        } catch (Exception e) {
            return null;
        }
    }

    public static @Nullable FileInputStream getInputStream(File file) {
        try {
            return new FileInputStream(file);
        } catch (Exception e) {
            return null;
        }
    }

    public static File getPath(Plugin plugin, String pathName) {
        return new File(plugin.getDataFolder()+File.separator+pathName);
    }

    public static File getFile(Plugin plugin, String fileName) {
        return getFile(plugin, null, fileName);
    }

    public static File getFile(Plugin plugin, String path, String fileName) {
        File p = (path == null || path.equals("")) ? plugin.getDataFolder() : new File(plugin.getDataFolder() + File.separator + path);
        if (!p.exists()) {
            p.mkdirs();
        }
        File file = new File(p + File.separator + fileName);
        if (!file.exists()) {
            try {
                saveResource(plugin, p+File.separator+fileName, false);
            } catch (Exception e) {
                try {
                    file.createNewFile();
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        }
        return file;
    }

    public static String getResource(Plugin plugin, String resourcePath) {
        if (resourcePath == null || resourcePath.equals("")) {
            throw new IllegalArgumentException("ResourcePath cannot be null or empty");
        }
        resourcePath = resourcePath.replace('\\', '/');
        InputStream in = plugin.getResource(resourcePath);
        if (in == null) {
            throw new IllegalArgumentException("The embedded resource '" + resourcePath + "' cannot be found in " + resourcePath);
        }

        try {
            ByteArrayOutputStream result = new ByteArrayOutputStream();
            byte[] buffer = new byte[1024];
            int length;
            while ((length = in.read(buffer)) != -1) {
                result.write(buffer, 0, length);
            }
            String str = result.toString(StandardCharsets.UTF_8.name());
            return str;
        } catch (Exception ex) {
            ex.printStackTrace();
        }
        return null;
    }

    public static void saveResource(Plugin plugin, String resourcePath, boolean replace) {
        if (resourcePath == null || resourcePath.equals("")) {
            throw new IllegalArgumentException("ResourcePath cannot be null or empty");
        }
        resourcePath = resourcePath.replace('\\', '/');
        InputStream in = plugin.getResource(resourcePath);
        if (in == null) {
            throw new IllegalArgumentException("The embedded resource '" + resourcePath + "' cannot be found in " + resourcePath);
        }
        File outFile = new File(plugin.getDataFolder(), resourcePath);
        int lastIndex = resourcePath.lastIndexOf(47);
        File outDir = new File(plugin.getDataFolder(), resourcePath.substring(0, (lastIndex >= 0) ? lastIndex : 0));
        if (!outDir.exists()) {
            outDir.mkdirs();
        }
        try {
            if (!outFile.exists() || replace) {
                OutputStream out = new FileOutputStream(outFile);
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                out.close();
                in.close();
            }
        } catch (IOException ex) {
            plugin.getLogger().log(Level.SEVERE, "Could not save " + outFile.getName() + " to " + outFile, ex);
        }
    }

    public static void saveResource(Plugin plugin, String resourcePath, String path, boolean replace) {
        if (resourcePath == null || resourcePath.equals("")) {
            throw new IllegalArgumentException("ResourcePath cannot be null or empty");
        }
        resourcePath = resourcePath.replace('\\', '/');
        InputStream in = plugin.getResource(resourcePath);
        if (in == null) {
            throw new IllegalArgumentException("The embedded resource '" + resourcePath + "' cannot be found in " + resourcePath);
        }
        File outFile = new File(path, resourcePath);
        int lastIndex = resourcePath.lastIndexOf(47);
        File outDir = new File(path, resourcePath.substring(0, (lastIndex >= 0) ? lastIndex : 0));
        if (!outDir.exists()) {
            outDir.mkdirs();
        }
        try {
            if (!outFile.exists() || replace) {
                OutputStream out = new FileOutputStream(outFile);
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                out.close();
                in.close();
            }
        } catch (IOException ex) {
            plugin.getLogger().log(Level.SEVERE, "Could not save " + outFile.getName() + " to " + outFile, ex);
        }
    }

    public static void saveResource(Plugin plugin, String resourcePath, String path, String rename, boolean replace) {
        if (resourcePath == null || resourcePath.equals("")) {
            throw new IllegalArgumentException("ResourcePath cannot be null or empty");
        }
        resourcePath = resourcePath.replace('\\', '/');
        InputStream in = plugin.getResource(resourcePath);
        if (in == null) {
            throw new IllegalArgumentException("The embedded resource '" + resourcePath + "' cannot be found in " + resourcePath);
        }
        File outFile = new File(path, rename);
        int lastIndex = resourcePath.lastIndexOf(47);
        File outDir = new File(path, resourcePath.substring(0, (lastIndex >= 0) ? lastIndex : 0));
        if (!outDir.exists()) {
            outDir.mkdirs();
        }
        try {
            if (!outFile.exists() || replace) {
                OutputStream out = new FileOutputStream(outFile);
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                out.close();
                in.close();
            }
        } catch (IOException ex) {
            plugin.getLogger().log(Level.SEVERE, "Could not save " + outFile.getName() + " to " + outFile, ex);
        }
    }

}
