package com.io.yutian.thewarskyblocklib.util;

import com.io.yutian.thewarskyblocklib.nms.item.NMSItemStack;
import com.io.yutian.thewarskyblocklib.nms.nbt.*;
import com.mojang.authlib.GameProfile;
import com.mojang.authlib.properties.Property;
import net.kyori.adventure.text.Component;
import org.bukkit.*;
import org.bukkit.block.banner.Pattern;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.EnchantmentStorageMeta;
import org.bukkit.inventory.meta.FireworkMeta;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.inventory.meta.SkullMeta;
import org.bukkit.potion.PotionEffect;

import java.lang.reflect.Field;
import java.util.*;

public class ItemStackBuilder {

    private ItemStack itemStack;

    private Material type;

    private int amount;

    private short data;

    private ItemMeta meta;

    private Component displayName;

    private List<Component> lore = new ArrayList<>();

    private Map<Enchantment, Integer> enchants = new HashMap<>();

    private Set<ItemFlag> itemFlags = new HashSet<>();

    private boolean unbreakable = false;

    private List<PotionEffect> potionEffects;
    private List<Pattern> patterns;
    private List<FireworkEffect> fireworkEffects;
    private String skullTextures;
    private UUID skullOwner;

    private NMSItemStack nmsItemStack;

    public ItemStackBuilder(Material material) {
        this.itemStack = new ItemStack(material);
        this.type = material;
        this.amount = 1;
        this.data = 0;
        this.meta = this.itemStack.getItemMeta();
        this.displayName = this.meta.displayName();
        this.lore = new ArrayList<>();
        this.enchants = new HashMap<>();
        this.itemFlags = new HashSet<>();
        this.potionEffects = new ArrayList<>();
        this.patterns = new ArrayList<>();
        this.fireworkEffects = new ArrayList<>();
        this.nmsItemStack = NMSItemStack.build(itemStack);
    }

    public ItemStackBuilder(ItemStack item) {
        this.itemStack = item;
        this.type = item.getType();
        this.amount = item.getAmount();
        this.data = item.getDurability();
        this.meta = item.getItemMeta();
        this.displayName = this.meta.displayName();
        this.lore = this.meta.lore() == null ? new ArrayList<>() : meta.lore();
        this.unbreakable = this.meta.isUnbreakable();
        this.enchants = this.meta.getEnchants();
        this.itemFlags = this.meta.getItemFlags();
        this.potionEffects = new ArrayList<>();
        this.patterns = new ArrayList<>();
        this.fireworkEffects = new ArrayList<>();
        this.nmsItemStack = NMSItemStack.build(itemStack);
    }

    public ItemStack build() {
        ItemStack item = nmsItemStack.getBukkitItemStack();
        item.setAmount(amount);
        if (enchants != null) {
            item.addEnchantments(enchants);
        }
        ItemMeta itemMeta = item.getItemMeta();
        if (displayName != null) {
            itemMeta.displayName(displayName);
        }
        if (lore != null) {
            itemMeta.lore(lore);
        }
        if (itemFlags != null) {
            for (ItemFlag flag : itemFlags) {
                itemMeta.addItemFlags(flag);
            }
        }
        itemMeta.setUnbreakable(unbreakable);
        if (fireworkEffects != null && fireworkEffects.size() > 0 && (type.equals(Material.FIREWORK_STAR) || type.equals(Material.FIREWORK_ROCKET))) {
            FireworkMeta fireworkMeta = (FireworkMeta) itemMeta;
            for (FireworkEffect effect : fireworkEffects) {
                fireworkMeta.addEffect(effect);
            }
        }
        if (skullTextures != null && type.equals(Material.PLAYER_HEAD)) {
            SkullMeta skullMeta = (SkullMeta) itemMeta;
            String uuid = "66d19f8d-b159-4034-9b86-3f75b6064629";

            GameProfile profile = new GameProfile(UUID.fromString(uuid), null);
            profile.getProperties().put("textures", new Property("textures", skullTextures));
            Field profileField;
            try {
                profileField = skullMeta.getClass().getDeclaredField("profile");
                profileField.setAccessible(true);
                profileField.set(skullMeta, profile);
            } catch (NullPointerException | NoSuchFieldException | IllegalArgumentException | IllegalAccessException e1) {
            }
        } else if (skullOwner != null && type.equals(Material.PLAYER_HEAD)) {
            SkullMeta skullMeta = (SkullMeta) itemMeta;
            OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(skullOwner);
            if (offlinePlayer != null) {
                skullMeta.setOwningPlayer(Bukkit.getOfflinePlayer(skullOwner));
            }
        }
        item.setItemMeta(itemMeta);
        return item;
    }

    public int getAmount() {
        return amount;
    }

    public Material getType() {
        return type;
    }

    public List<Component> getLore() {
        return lore;
    }

    public Map<Enchantment, Integer> getEnchants() {
        return enchants;
    }

    public Set<ItemFlag> getItemFlags() {
        return itemFlags;
    }

    public List<PotionEffect> getPotionEffects() {
        return potionEffects;
    }

    public List<FireworkEffect> getFireworkEffects() {
        return fireworkEffects;
    }

    public boolean isUnbreakable() {
        return unbreakable;
    }

    public List<Pattern> getPatterns() {
        return patterns;
    }

    public String getSkullTextures() {
        return skullTextures;
    }

    public Component getDisplayName() {
        return displayName;
    }

    public short getData() {
        return data;
    }

    public UUID getSkullOwner() {
        return skullOwner;
    }

    public ItemStackBuilder setItemStack(ItemStack item) {
        this.itemStack = item;
        return this;
    }

    public ItemStackBuilder setType(Material type) {
        this.type = type;
        return this;
    }

    public ItemStackBuilder setAmount(int amount) {
        this.amount = amount;
        return this;
    }

    public ItemStackBuilder setData(int data) {
        if (data > 255) {
            data = 255;
        }
        this.data = (short) data;
        return this;
    }

    public ItemStackBuilder setData(short data) {
        this.data = data;
        return this;
    }

    public ItemStackBuilder setItemMeta(ItemMeta meta) {
        this.meta = meta;
        return this;
    }

    public ItemStackBuilder setDisplayName(String name) {
        this.displayName = Component.text(name);
        return this;
    }

    public ItemStackBuilder setDisplayName(Component name) {
        this.displayName = name;
        return this;
    }

    public ItemStackBuilder setUnbreakable(boolean flag) {
        this.unbreakable = flag;
        return this;
    }

    public ItemStackBuilder setLore(List<Component> loreList) {
        this.lore = loreList;
        return this;
    }

    public ItemStackBuilder setStringLore(List<String> loreList) {
        List<Component> list = new ArrayList<>();
        loreList.forEach(s -> {
            list.add(Component.text(s));
        });
        this.lore = list;
        return this;
    }

    public ItemStackBuilder setLore(String... lores) {
        List<Component> list = new ArrayList<>();
        for (String s : lores) {
            list.add(Component.text(s));
        }
        this.lore = list;
        return this;
    }

    public ItemStackBuilder setLore(Component... lores) {
        this.lore = new ArrayList<>(Arrays.asList(lores));
        return this;
    }

    public ItemStackBuilder addLore(String lore) {
        this.lore.add(Component.text(lore));
        return this;
    }

    public ItemStackBuilder addLore(String... lores) {
        List<Component> list = new ArrayList<>();
        for (String s : lores) {
            list.add(Component.text(s));
        }
        this.lore.addAll(list);
        return this;
    }

    public ItemStackBuilder addLore(Component lore) {
        this.lore.add(lore);
        return this;
    }

    public ItemStackBuilder addLore(Component... lores) {
        this.lore.addAll(Arrays.asList(lores));
        return this;
    }

    public ItemStackBuilder addLore(List<Component> lores) {
        for (Component lore : lores) {
            this.lore.add(lore);
        }
        return this;
    }

    public ItemStackBuilder addEnchant(Enchantment ench, int level) {
        this.enchants.put(ench, level);
        return this;
    }

    public ItemStackBuilder removeEnchant(Enchantment ench) {
        this.enchants.remove(ench);
        return this;
    }

    public ItemStackBuilder setEnchants(Map<Enchantment, Integer> enchants) {
        this.enchants = enchants;
        return this;
    }

    public ItemStackBuilder setEnchants(List<String> enchants) {
        this.enchants = getEnchantsFromList(enchants);
        return this;
    }

    public ItemStackBuilder setItemFlags(Set<ItemFlag> itemFlags) {
        this.itemFlags = itemFlags;
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag[] itemFlags) {
        this.itemFlags.addAll(Arrays.asList(itemFlags));
        return this;
    }

    public ItemStackBuilder addItemFlags(ItemFlag itemFlag) {
        this.itemFlags.add(itemFlag);
        return this;
    }

    public ItemStackBuilder addPotionEffect(PotionEffect potionEffect) {
        this.potionEffects.add(potionEffect);
        return this;
    }

    public ItemStackBuilder addPattern(Pattern pattern) {
        this.patterns.add(pattern);
        return this;
    }

    public ItemStackBuilder addFireworkEffect(FireworkEffect fireworkEffect) {
        this.fireworkEffects.add(fireworkEffect);
        return this;
    }

    public ItemStackBuilder setSkullTextures(String skullTextures) {
        this.skullTextures = skullTextures;
        return this;
    }

    public ItemStackBuilder setSkullOwner(UUID skullOwner) {
        this.skullOwner = skullOwner;
        return this;
    }

    public ItemStackBuilder setItemHandler(String id) {
        return setNBTTagString("item_handler", id);
    }

    public NMSItemStack getNMSItemStack() {
        return nmsItemStack;
    }

    public NBTBase getNBTTag(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.get(key);
    }

    public NBTCompound getNBTCompound(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getCompound(key);
    }

    public NBTList getNBTTagList(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getList(key);
    }

    public String getNBTTagString(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getString(key);
    }

    public boolean getNBTTagBoolean(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getByte(key) == 1 ? true : false;
    }

    public int getNBTTagInt(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getInt(key);
    }

    public NBTIntArray getNBTTagIntArray(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return (NBTIntArray) nbtCompound.get(key);
    }

    public double getNBTTagDouble(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getDouble(key);
    }

    public long getNBTTagLong(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getLong(key);
    }

    public short getNBTTagShort(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getShort(key);
    }

    public byte getNBTTagByte(String key) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        return nbtCompound.getByte(key);
    }

    public ItemStackBuilder setNBTTag(String key, NBTBase value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, value);
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTCompound(String key, NBTCompound compound) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, compound);
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagList(String key, NBTBase... value) {
        NBTList list = new NBTList();
        for (NBTBase nb : value) {
            list.add(nb);
        }
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, list);
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagString(String key, String value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTString(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagBoolean(String key, boolean value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTByte((byte) (value ? 1 : 0)));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagInt(String key, int value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTInt(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagIntArray(String key, int[] value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTIntArray(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagDouble(String key, double value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTDouble(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagLong(String key, long value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTLong(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagFloat(String key, float value) {
        this.nmsItemStack.getTag().put(key, new NBTFloat(value));
        return this;
    }

    public ItemStackBuilder setNBTTagShort(String key, short value) {
        this.nmsItemStack.getTag().put(key, new NBTShort(value));
        return this;
    }

    public ItemStackBuilder setNBTTagByte(String key, byte value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTByte(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    public ItemStackBuilder setNBTTagByteArray(String key, byte[] value) {
        NBTCompound nbtCompound = this.nmsItemStack.getTag();
        nbtCompound.put(key, new NBTByteArray(value));
        this.nmsItemStack.setTag(nbtCompound);
        return this;
    }

    protected Map<Enchantment, Integer> getEnchantsFromList(List<String> enchants) {
        Map<Enchantment, Integer> map = new HashMap<>();
        if (enchants == null) {
            return map;
        }
        for (String s : enchants) {
            if (s.contains(":")) {
                String[] part = s.split(":");
                Enchantment en = Enchantment.getByName(part[0]);
                if (en == null) {
                    continue;
                }
                map.put(en, Integer.parseInt(part[1]));
            }
        }
        return map;
    }

    protected static ItemStack setEnchants(ItemStack stack, List<String> enchants) {
        if (enchants == null) {
            return stack;
        }
        for (String s : enchants) {
            if (s.contains(":")) {
                String[] part = s.split(":");
                Enchantment en = Enchantment.getByName(part[0]);
                if (en == null) {
                    continue;
                }
                if (stack.getType() != Material.ENCHANTED_BOOK) {
                    stack.addUnsafeEnchantment(en, Integer.parseInt(part[1]));
                } else {
                    EnchantmentStorageMeta esm = (EnchantmentStorageMeta) stack.getItemMeta();
                    esm.addStoredEnchant(en, Integer.parseInt(part[1]), true);
                    stack.setItemMeta((ItemMeta) esm);
                }
            }
        }
        return stack;
    }

    private void setField(Object instance, String name, Object value) throws ReflectiveOperationException {
        Field field = instance.getClass().getDeclaredField(name);
        field.setAccessible(true);
        field.set(instance, value);
    }

}
