package com.io.yutian.thewarskyblocklib.util;

import net.kyori.adventure.text.Component;
import org.bukkit.Material;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.List;

public class ItemStackUtil {

    public static Component getName(ItemStack itemStack) {
        ItemMeta meta = itemStack.getItemMeta();
        if (meta.hasDisplayName()) {
            return meta.displayName();
        } else {
            return Component.text(itemStack.getType().toString());
        }
    }

    public static ItemStack setDisplayName(ItemStack itemStack, Component component) {
        ItemMeta meta = itemStack.getItemMeta();
        meta.displayName(component);
        ItemStack itemStack1 = itemStack.clone();
        itemStack1.setItemMeta(meta);
        return itemStack1;
    }

    public static List<Component> getLores(ItemStack itemStack) {
        ItemMeta meta = itemStack.getItemMeta();
        return meta.lore();
    }

    public static ItemStack setLores(ItemStack itemStack, List<Component> lores) {
        ItemMeta meta = itemStack.getItemMeta();
        meta.lore(lores);
        ItemStack itemStack1 = itemStack.clone();
        itemStack1.setItemMeta(meta);
        return itemStack1;
    }

    public static ItemStack addLores(ItemStack itemStack, Component... lores) {
        ItemMeta meta = itemStack.getItemMeta();
        List<Component> components = meta.lore();
        for (Component component : lores) {
            components.add(component);
        }
        meta.lore(components);
        ItemStack itemStack1 = itemStack.clone();
        itemStack1.setItemMeta(meta);
        return itemStack1;
    }

    public static ItemStack addItemFlays(ItemStack itemStack, ItemFlag... itemFlags) {
        ItemMeta meta = itemStack.getItemMeta();
        meta.addItemFlags(itemFlags);
        ItemStack itemStack1 = itemStack.clone();
        itemStack1.setItemMeta(meta);
        return itemStack1;
    }

    public static boolean isAir(ItemStack itemStack) {
        if (itemStack == null) {
            return true;
        }
        if (itemStack.getType().equals(Material.AIR)) {
            return true;
        }
        return false;
    }

    public static boolean hasLore(ItemStack itemStack, String lore) {
        return hasLore(itemStack, lore, true);
    }

    public static boolean hasLore(ItemStack itemStack, String lore, boolean fuzzyMatch) {
        if (itemStack == null || itemStack.getType().equals(Material.AIR)) {
            return false;
        }
        ItemMeta meta = itemStack.getItemMeta();
        if (!meta.hasLore()) {
            return false;
        }
        List<String> list = meta.getLore();
        for (String l : list) {
            if (fuzzyMatch) {
                if (l.contains(lore)) {
                    return true;
                }
            } else {
                if (l.equalsIgnoreCase(lore)) {
                    return true;
                }
            }
        }
        return false;
    }

}
